# -*- coding: utf-8 -*-

#
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_DialogModulationParameters(object):
    def setupUi(self, DialogModulationParameters):
        DialogModulationParameters.setObjectName("DialogModulationParameters")
        DialogModulationParameters.resize(303, 286)
        DialogModulationParameters.setModal(True)
        self.verticalLayout = QtWidgets.QVBoxLayout(DialogModulationParameters)
        self.verticalLayout.setObjectName("verticalLayout")
        self.tblSymbolParameters = QtWidgets.QTableWidget(DialogModulationParameters)
        self.tblSymbolParameters.setShowGrid(False)
        self.tblSymbolParameters.setRowCount(2)
        self.tblSymbolParameters.setObjectName("tblSymbolParameters")
        self.tblSymbolParameters.setColumnCount(2)
        item = QtWidgets.QTableWidgetItem()
        self.tblSymbolParameters.setHorizontalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        self.tblSymbolParameters.setHorizontalHeaderItem(1, item)
        self.tblSymbolParameters.horizontalHeader().setVisible(True)
        self.tblSymbolParameters.verticalHeader().setVisible(False)
        self.verticalLayout.addWidget(self.tblSymbolParameters)
        self.buttonBox = QtWidgets.QDialogButtonBox(DialogModulationParameters)
        self.buttonBox.setStandardButtons(
            QtWidgets.QDialogButtonBox.Cancel | QtWidgets.QDialogButtonBox.Ok
        )
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout.addWidget(self.buttonBox)

        self.retranslateUi(DialogModulationParameters)

    def retranslateUi(self, DialogModulationParameters):
        _translate = QtCore.QCoreApplication.translate
        DialogModulationParameters.setWindowTitle(
            _translate("DialogModulationParameters", "Modulation Parameters")
        )
        item = self.tblSymbolParameters.horizontalHeaderItem(0)
        item.setText(_translate("DialogModulationParameters", "Symbol"))
        item = self.tblSymbolParameters.horizontalHeaderItem(1)
        item.setText(_translate("DialogModulationParameters", "Amplitude"))



from collections import OrderedDict
from multiprocessing import Array

import numpy as np

from urh.dev.native.Device import Device
from urh.dev.native.lib import bladerf
from multiprocessing.connection import Connection


class BladeRF(Device):
    SYNC_RX_CHUNK_SIZE = 16384
    SYNC_TX_CHUNK_SIZE = 16384

    DEVICE_LIB = bladerf
    ASYNCHRONOUS = False
    DEVICE_METHODS = Device.DEVICE_METHODS.copy()
    DEVICE_METHODS.update(
        {
            Device.Command.SET_RF_GAIN.name: "set_gain",
            Device.Command.SET_CHANNEL_INDEX.name: "set_channel",
            Device.Command.SET_BIAS_TEE_ENABLED.name: "set_bias_tee",
        }
    )

    DATA_TYPE = np.int16

    @classmethod
    def get_device_list(cls):
        return bladerf.get_device_list()

    @classmethod
    def adapt_num_read_samples_to_sample_rate(cls, sample_rate):
        cls.SYNC_RX_CHUNK_SIZE = 16384 * int(sample_rate / 1e6)

    @classmethod
    def setup_device(cls, ctrl_connection: Connection, device_identifier):
        if not device_identifier:
            device_identifier = ""

        ret = bladerf.open(device_identifier)
        if not device_identifier:
            ctrl_connection.send("OPEN:" + str(ret))
        else:
            ctrl_connection.send("OPEN ({}):{}".format(device_identifier, ret))

        ctrl_connection.send(
            "If you experience problems, make sure you place a rbf file matching your device"
            " at the correct location. See http://www.nuand.com/fpga_images/"
            " and https://github.com/Nuand/bladeRF/wiki/FPGA-Autoloading"
        )

        return ret == 0

    @classmethod
    def init_device(
        cls, ctrl_connection: Connection, is_tx: bool, parameters: OrderedDict
    ) -> bool:
        bladerf.set_tx(is_tx)
        return super().init_device(ctrl_connection, is_tx, parameters)

    @classmethod
    def shutdown_device(cls, ctrl_connection, is_tx: bool):
        ret = bladerf.close()
        ctrl_connection.send("CLOSE:" + str(ret))
        return True

    @classmethod
    def prepare_sync_receive(cls, ctrl_connection: Connection):
        ctrl_connection.send("Initializing BladeRF..")
        ret = bladerf.prepare_sync()
        return ret

    @classmethod
    def receive_sync(cls, data_conn: Connection):
        bladerf.receive_sync(data_conn, cls.SYNC_RX_CHUNK_SIZE)

    @classmethod
    def prepare_sync_send(cls, ctrl_connection: Connection):
        ctrl_connection.send("Initializing BladeRF...")
        ret = bladerf.prepare_sync()
        return ret

    @classmethod
    def send_sync(cls, data):
        bladerf.send_sync(data)

    def __init__(
        self,
        center_freq,
        sample_rate,
        bandwidth,
        gain,
        if_gain=1,
        baseband_gain=1,
        resume_on_full_receive_buffer=False,
    ):
        super().__init__(
            center_freq=center_freq,
            sample_rate=sample_rate,
            bandwidth=bandwidth,
            gain=gain,
            if_gain=if_gain,
            baseband_gain=baseband_gain,
            resume_on_full_receive_buffer=resume_on_full_receive_buffer,
        )
        self.success = 0

    @property
    def has_multi_device_support(self):
        return True

    @property
    def device_parameters(self):
        return OrderedDict(
            [
                (self.Command.SET_CHANNEL_INDEX.name, self.channel_index),
                (self.Command.SET_FREQUENCY.name, self.frequency),
                (self.Command.SET_SAMPLE_RATE.name, self.sample_rate),
                (self.Command.SET_BANDWIDTH.name, self.bandwidth),
                (self.Command.SET_RF_GAIN.name, self.gain),
                (self.Command.SET_BIAS_TEE_ENABLED.name, self.bias_tee_enabled),
                ("identifier", self.device_serial),
            ]
        )

    @staticmethod
    def bytes_to_iq(buffer) -> np.ndarray:
        return np.frombuffer(buffer, dtype=np.int16).reshape((-1, 2), order="C") << 4

    @staticmethod
    def iq_to_bytes(iq_samples: np.ndarray):
        arr = Array("h", 2 * len(iq_samples), lock=False)
        numpy_view = np.frombuffer(arr, dtype=np.int16)
        numpy_view[:] = iq_samples.flatten(order="C") >> 4
        return arr



# -*- coding: utf-8 -*-

#
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_GeneratorTab(object):
    def setupUi(self, GeneratorTab):
        GeneratorTab.setObjectName("GeneratorTab")
        GeneratorTab.resize(1287, 774)
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(GeneratorTab)
        self.verticalLayout_3.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_3.setSpacing(0)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.scrollArea = QtWidgets.QScrollArea(GeneratorTab)
        self.scrollArea.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.scrollArea.setWidgetResizable(True)
        self.scrollArea.setObjectName("scrollArea")
        self.scrollAreaWidgetContents = QtWidgets.QWidget()
        self.scrollAreaWidgetContents.setGeometry(QtCore.QRect(0, 0, 1287, 774))
        self.scrollAreaWidgetContents.setObjectName("scrollAreaWidgetContents")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.scrollAreaWidgetContents)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.splitter = QtWidgets.QSplitter(self.scrollAreaWidgetContents)
        self.splitter.setStyleSheet(
            "QSplitter::handle:horizontal {\n"
            "margin: 4px 0px;\n"
            "    background-color: qlineargradient(x1:0, y1:0, x2:0, y2:1, \n"
            "stop:0 rgba(255, 255, 255, 0), \n"
            "stop:0.5 rgba(100, 100, 100, 100), \n"
            "stop:1 rgba(255, 255, 255, 0));\n"
            "image: url(:/icons/icons/splitter_handle_vertical.svg);\n"
            "}"
        )
        self.splitter.setOrientation(QtCore.Qt.Horizontal)
        self.splitter.setHandleWidth(6)
        self.splitter.setObjectName("splitter")
        self.layoutWidget_2 = QtWidgets.QWidget(self.splitter)
        self.layoutWidget_2.setObjectName("layoutWidget_2")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.layoutWidget_2)
        self.verticalLayout.setContentsMargins(11, 11, 11, 11)
        self.verticalLayout.setObjectName("verticalLayout")
        self.tabWidget = QtWidgets.QTabWidget(self.layoutWidget_2)
        sizePolicy = QtWidgets.QSizePolicy(
            QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding
        )
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.tabWidget.sizePolicy().hasHeightForWidth())
        self.tabWidget.setSizePolicy(sizePolicy)
        self.tabWidget.setStyleSheet("QTabWidget::pane { border: 0; }")
        self.tabWidget.setTabPosition(QtWidgets.QTabWidget.North)
        self.tabWidget.setTabShape(QtWidgets.QTabWidget.Rounded)
        self.tabWidget.setObjectName("tabWidget")
        self.tab_proto = QtWidgets.QWidget()
        self.tab_proto.setObjectName("tab_proto")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.tab_proto)
        self.verticalLayout_4.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.treeProtocols = GeneratorTreeView(self.tab_proto)
        sizePolicy = QtWidgets.QSizePolicy(
            QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Expanding
        )
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(
            self.treeProtocols.sizePolicy().hasHeightForWidth()
        )
        self.treeProtocols.setSizePolicy(sizePolicy)
        self.treeProtocols.setObjectName("treeProtocols")
        self.treeProtocols.header().setDefaultSectionSize(57)
        self.verticalLayout_4.addWidget(self.treeProtocols)
        self.tabWidget.addTab(self.tab_proto, "")
        self.tab_pauses = QtWidgets.QWidget()
        self.tab_pauses.setObjectName("tab_pauses")
        self.gridLayout_5 = QtWidgets.QGridLayout(self.tab_pauses)
        self.gridLayout_5.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_5.setObjectName("gridLayout_5")
        self.lWPauses = GeneratorListWidget(self.tab_pauses)
        sizePolicy = QtWidgets.QSizePolicy(
            QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding
        )
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lWPauses.sizePolicy().hasHeightForWidth())
        self.lWPauses.setSizePolicy(sizePolicy)
        self.lWPauses.setEditTriggers(
            QtWidgets.QAbstractItemView.DoubleClicked
            | QtWidgets.QAbstractItemView.EditKeyPressed
        )
        self.lWPauses.setProperty("showDropIndicator", False)
        self.lWPauses.setDragDropMode(QtWidgets.QAbstractItemView.NoDragDrop)
        self.lWPauses.setObjectName("lWPauses")
        self.gridLayout_5.addWidget(self.lWPauses, 0, 0, 1, 2)
        self.tabWidget.addTab(self.tab_pauses, "")
        self.tab_fuzzing = QtWidgets.QWidget()
        self.tab_fuzzing.setObjectName("tab_fuzzing")
        self.verticalLayout_9 = QtWidgets.QVBoxLayout(self.tab_fuzzing)
        self.verticalLayout_9.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_9.setSpacing(6)
        self.verticalLayout_9.setObjectName("verticalLayout_9")
        self.listViewProtoLabels = GeneratorListView(self.tab_fuzzing)
        sizePolicy = QtWidgets.QSizePolicy(
            QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding
        )
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(
            self.listViewProtoLabels.sizePolicy().hasHeightForWidth()
        )
        self.listViewProtoLabels.setSizePolicy(sizePolicy)
        self.listViewProtoLabels.setEditTriggers(
            QtWidgets.QAbstractItemView.EditKeyPressed
        )
        self.listViewProtoLabels.setObjectName("listViewProtoLabels")
        self.verticalLayout_9.addWidget(self.listViewProtoLabels)
        self.groupBox = QtWidgets.QGroupBox(self.tab_fuzzing)
        self.groupBox.setObjectName("groupBox")
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout(self.groupBox)
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.stackedWidgetFuzzing = QtWidgets.QStackedWidget(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(
            QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Maximum
        )
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(
            self.stackedWidgetFuzzing.sizePolicy().hasHeightForWidth()
        )
        self.stackedWidgetFuzzing.setSizePolicy(sizePolicy)
        self.stackedWidgetFuzzing.setObjectName("stackedWidgetFuzzing")
        self.pageFuzzingUI = QtWidgets.QWidget()
        self.pageFuzzingUI.setObjectName("pageFuzzingUI")
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout(self.pageFuzzingUI)
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.btnFuzz = QtWidgets.QPushButton(self.pageFuzzingUI)
        self.btnFuzz.setObjectName("btnFuzz")
        self.horizontalLayout_4.addWidget(self.btnFuzz)
        self.rBSuccessive = QtWidgets.QRadioButton(self.pageFuzzingUI)
        self.rBSuccessive.setChecked(True)
        self.rBSuccessive.setObjectName("rBSuccessive")
        self.horizontalLayout_4.addWidget(self.rBSuccessive)
        self.rbConcurrent = QtWidgets.QRadioButton(self.pageFuzzingUI)
        self.rbConcurrent.setObjectName("rbConcurrent")
        self.horizontalLayout_4.addWidget(self.rbConcurrent)
        self.rBExhaustive = QtWidgets.QRadioButton(self.pageFuzzingUI)
        self.rBExhaustive.setObjectName("rBExhaustive")
        self.horizontalLayout_4.addWidget(self.rBExhaustive)
        self.stackedWidgetFuzzing.addWidget(self.pageFuzzingUI)
        self.pageFuzzingProgressBar = QtWidgets.QWidget()
        self.pageFuzzingProgressBar.setObjectName("pageFuzzingProgressBar")
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout(self.pageFuzzingProgressBar)
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        self.progressBarFuzzing = QtWidgets.QProgressBar(self.pageFuzzingProgressBar)
        self.progressBarFuzzing.setProperty("value", 24)
        self.progressBarFuzzing.setObjectName("progressBarFuzzing")
        self.horizontalLayout_7.addWidget(self.progressBarFuzzing)
        self.stackedWidgetFuzzing.addWidget(self.pageFuzzingProgressBar)
        self.horizontalLayout_6.addWidget(self.stackedWidgetFuzzing)
        self.verticalLayout_9.addWidget(self.groupBox)
        self.tabWidget.addTab(self.tab_fuzzing, "")
        self.verticalLayout.addWidget(self.tabWidget)
        self.line_2 = QtWidgets.QFrame(self.layoutWidget_2)
        self.line_2.setFrameShape(QtWidgets.QFrame.HLine)
        self.line_2.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line_2.setObjectName("line_2")
        self.verticalLayout.addWidget(self.line_2)
        self.gridLayout_6 = QtWidgets.QGridLayout()
        self.gridLayout_6.setContentsMargins(0, -1, 0, -1)
        self.gridLayout_6.setObjectName("gridLayout_6")
        self.modulationLayout_2 = QtWidgets.QGridLayout()
        self.modulationLayout_2.setObjectName("modulationLayout_2")
        self.lCarrierFreqValue = QtWidgets.QLabel(self.layoutWidget_2)
        self.lCarrierFreqValue.setObjectName("lCarrierFreqValue")
        self.modulationLayout_2.addWidget(self.lCarrierFreqValue, 1, 1, 1, 1)
        self.lModType = QtWidgets.QLabel(self.layoutWidget_2)
        self.lModType.setObjectName("lModType")
        self.modulationLayout_2.addWidget(self.lModType, 1, 2, 1, 1)
        self.lModTypeValue = QtWidgets.QLabel(self.layoutWidget_2)
        self.lModTypeValue.setObjectName("lModTypeValue")
        self.modulationLayout_2.addWidget(self.lModTypeValue, 1, 3, 1, 1)
        self.label_carrier_phase = QtWidgets.QLabel(self.layoutWidget_2)
        self.label_carrier_phase.setObjectName("label_carrier_phase")
        self.modulationLayout_2.addWidget(self.label_carrier_phase, 2, 0, 1, 1)
        self.lCarrierPhaseValue = QtWidgets.QLabel(self.layoutWidget_2)
        self.lCarrierPhaseValue.setObjectName("lCarrierPhaseValue")
        self.modulationLayout_2.addWidget(self.lCarrierPhaseValue, 2, 1, 1, 1)
        self.lBitLength = QtWidgets.QLabel(self.layoutWidget_2)
        self.lBitLength.setObjectName("lBitLength")
        self.modulationLayout_2.addWidget(self.lBitLength, 3, 0, 1, 1)
        self.lBitLenValue = QtWidgets.QLabel(self.layoutWidget_2)
        self.lBitLenValue.setObjectName("lBitLenValue")
        self.modulationLayout_2.addWidget(self.lBitLenValue, 3, 1, 1, 1)
        self.lEncoding = QtWidgets.QLabel(self.layoutWidget_2)
        self.lEncoding.setObjectName("lEncoding")
        self.modulationLayout_2.addWidget(self.lEncoding, 0, 0, 1, 1)
        self.lEncodingValue = QtWidgets.QLabel(self.layoutWidget_2)
        sizePolicy = QtWidgets.QSizePolicy(
            QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred
        )
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(
            self.lEncodingValue.sizePolicy().hasHeightForWidth()
        )
        self.lEncodingValue.setSizePolicy(sizePolicy)
        self.lEncodingValue.setObjectName("lEncodingValue")
        self.modulationLayout_2.addWidget(self.lEncodingValue, 0, 1, 1, 1)
        self.lSampleRate = QtWidgets.QLabel(self.layoutWidget_2)
        self.lSampleRate.setObjectName("lSampleRate")
        self.modulationLayout_2.addWidget(self.lSampleRate, 0, 2, 1, 1)
        self.lSampleRateValue = QtWidgets.QLabel(self.layoutWidget_2)
        self.lSampleRateValue.setObjectName("lSampleRateValue")
        self.modulationLayout_2.addWidget(self.lSampleRateValue, 0, 3, 1, 1)
        self.lCarrierFrequency = QtWidgets.QLabel(self.layoutWidget_2)
        self.lCarrierFrequency.setObjectName("lCarrierFrequency")
        self.modulationLayout_2.addWidget(self.lCarrierFrequency, 1, 0, 1, 1)
        self.labelParameterValues = ElidedLabel(self.layoutWidget_2)
        self.labelParameterValues.setObjectName("labelParameterValues")
        self.modulationLayout_2.addWidget(self.labelParameterValues, 3, 3, 1, 1)
        self.lParamCaption = QtWidgets.QLabel(self.layoutWidget_2)
        self.lParamCaption.setObjectName("lParamCaption")
        self.modulationLayout_2.addWidget(self.lParamCaption, 3, 2, 1, 1)
        self.label = QtWidgets.QLabel(self.layoutWidget_2)
        self.label.setObjectName("label")
        self.modulationLayout_2.addWidget(self.label, 2, 2, 1, 1)
        self.labelBitsPerSymbol = QtWidgets.QLabel(self.layoutWidget_2)
        self.labelBitsPerSymbol.setObjectName("labelBitsPerSymbol")
        self.modulationLayout_2.addWidget(self.labelBitsPerSymbol, 2, 3, 1, 1)
        self.gridLayout_6.addLayout(self.modulationLayout_2, 0, 0, 1, 3)
        self.line = QtWidgets.QFrame(self.layoutWidget_2)
        self.line.setFrameShape(QtWidgets.QFrame.HLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.gridLayout_6.addWidget(self.line, 1, 0, 1, 3)
        self.cBoxModulations = QtWidgets.QComboBox(self.layoutWidget_2)
        self.cBoxModulations.setObjectName("cBoxModulations")
        self.cBoxModulations.addItem("")
        self.gridLayout_6.addWidget(self.cBoxModulations, 2, 1, 1, 1)
        self.prBarGeneration = QtWidgets.QProgressBar(self.layoutWidget_2)
        sizePolicy = QtWidgets.QSizePolicy(
            QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed
        )
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(
            self.prBarGeneration.sizePolicy().hasHeightForWidth()
        )
        self.prBarGeneration.setSizePolicy(sizePolicy)
        self.prBarGeneration.setProperty("value", 0)
        self.prBarGeneration.setObjectName("prBarGeneration")
        self.gridLayout_6.addWidget(self.prBarGeneration, 5, 0, 1, 1)
        self.btnSend = QtWidgets.QPushButton(self.layoutWidget_2)
        self.btnSend.setEnabled(False)
        icon = QtGui.QIcon.fromTheme("media-playback-start")
        self.btnSend.setIcon(icon)
        self.btnSend.setObjectName("btnSend")
        self.gridLayout_6.addWidget(self.btnSend, 5, 2, 1, 1)
        self.btnEditModulation = QtWidgets.QPushButton(self.layoutWidget_2)
        self.btnEditModulation.setObjectName("btnEditModulation")
        self.gridLayout_6.addWidget(self.btnEditModulation, 2, 2, 1, 1)
        self.lModulation = QtWidgets.QLabel(self.layoutWidget_2)
        self.lModulation.setObjectName("lModulation")
        self.gridLayout_6.addWidget(self.lModulation, 2, 0, 1, 1)
        self.btnGenerate = QtWidgets.QPushButton(self.layoutWidget_2)
        self.btnGenerate.setEnabled(False)
        icon = QtGui.QIcon.fromTheme("document-new")
        self.btnGenerate.setIcon(icon)
        self.btnGenerate.setObjectName("btnGenerate")
        self.gridLayout_6.addWidget(self.btnGenerate, 5, 1, 1, 1)
        self.verticalLayout.addLayout(self.gridLayout_6)
        self.layoutWidget = QtWidgets.QWidget(self.splitter)
        self.layoutWidget.setObjectName("layoutWidget")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.layoutWidget)
        self.gridLayout_2.setContentsMargins(11, 11, 11, 11)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.tableMessages = GeneratorTableView(self.layoutWidget)
        self.tableMessages.setAcceptDrops(True)
        self.tableMessages.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.tableMessages.setDragEnabled(False)
        self.tableMessages.setDragDropOverwriteMode(False)
        self.tableMessages.setDragDropMode(QtWidgets.QAbstractItemView.DropOnly)
        self.tableMessages.setDefaultDropAction(QtCore.Qt.CopyAction)
        self.tableMessages.setAlternatingRowColors(True)
        self.tableMessages.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectItems)
        self.tableMessages.setVerticalScrollMode(
            QtWidgets.QAbstractItemView.ScrollPerPixel
        )
        self.tableMessages.setHorizontalScrollMode(
            QtWidgets.QAbstractItemView.ScrollPerPixel
        )
        self.tableMessages.setShowGrid(False)
        self.tableMessages.setObjectName("tableMessages")
        self.tableMessages.horizontalHeader().setHighlightSections(False)
        self.tableMessages.verticalHeader().setHighlightSections(False)
        self.gridLayout_2.addWidget(self.tableMessages, 1, 0, 1, 8)
        self.btnNetworkSDRSend = QtWidgets.QPushButton(self.layoutWidget)
        icon = QtGui.QIcon.fromTheme("network-wired")
        self.btnNetworkSDRSend.setIcon(icon)
        self.btnNetworkSDRSend.setCheckable(True)
        self.btnNetworkSDRSend.setObjectName("btnNetworkSDRSend")
        self.gridLayout_2.addWidget(self.btnNetworkSDRSend, 2, 0, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(
            38, 22, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum
        )
        self.gridLayout_2.addItem(spacerItem, 2, 4, 1, 2)
        self.cbViewType = QtWidgets.QComboBox(self.layoutWidget)
        self.cbViewType.setObjectName("cbViewType")
        self.cbViewType.addItem("")
        self.cbViewType.addItem("")
        self.cbViewType.addItem("")
        self.gridLayout_2.addWidget(self.cbViewType, 2, 7, 1, 1)
        self.lViewType = QtWidgets.QLabel(self.layoutWidget)
        sizePolicy = QtWidgets.QSizePolicy(
            QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred
        )
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lViewType.sizePolicy().hasHeightForWidth())
        self.lViewType.setSizePolicy(sizePolicy)
        self.lViewType.setObjectName("lViewType")
        self.gridLayout_2.addWidget(self.lViewType, 2, 6, 1, 1)
        self.btnRfCatSend = QtWidgets.QPushButton(self.layoutWidget)
        icon = QtGui.QIcon.fromTheme("network-wireless")
        self.btnRfCatSend.setIcon(icon)
        self.btnRfCatSend.setObjectName("btnRfCatSend")
        self.gridLayout_2.addWidget(self.btnRfCatSend, 2, 1, 1, 1)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setContentsMargins(-1, 0, -1, 0)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.labelGeneratedData = QtWidgets.QLabel(self.layoutWidget)
        sizePolicy = QtWidgets.QSizePolicy(
            QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred
        )
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(
            self.labelGeneratedData.sizePolicy().hasHeightForWidth()
        )
        self.labelGeneratedData.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.labelGeneratedData.setFont(font)
        self.labelGeneratedData.setAlignment(QtCore.Qt.AlignCenter)
        self.labelGeneratedData.setObjectName("labelGeneratedData")
        self.horizontalLayout.addWidget(self.labelGeneratedData)
        self.btnSave = QtWidgets.QToolButton(self.layoutWidget)
        sizePolicy = QtWidgets.QSizePolicy(
            QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred
        )
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.btnSave.sizePolicy().hasHeightForWidth())
        self.btnSave.setSizePolicy(sizePolicy)
        icon = QtGui.QIcon.fromTheme("document-save")
        self.btnSave.setIcon(icon)
        self.btnSave.setObjectName("btnSave")
        self.horizontalLayout.addWidget(self.btnSave)
        self.btnOpen = QtWidgets.QToolButton(self.layoutWidget)
        sizePolicy = QtWidgets.QSizePolicy(
            QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred
        )
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.btnOpen.sizePolicy().hasHeightForWidth())
        self.btnOpen.setSizePolicy(sizePolicy)
        icon = QtGui.QIcon.fromTheme("document-open")
        self.btnOpen.setIcon(icon)
        self.btnOpen.setObjectName("btnOpen")
        self.horizontalLayout.addWidget(self.btnOpen)
        self.gridLayout_2.addLayout(self.horizontalLayout, 0, 0, 1, 8)
        self.lEstimatedTime = QtWidgets.QLabel(self.layoutWidget)
        self.lEstimatedTime.setObjectName("lEstimatedTime")
        self.gridLayout_2.addWidget(self.lEstimatedTime, 2, 3, 1, 1)
        self.btnFZSave = QtWidgets.QPushButton(self.layoutWidget)
        icon = QtGui.QIcon.fromTheme("document-save-as")
        self.btnFZSave.setIcon(icon)
        self.btnFZSave.setObjectName("btnFZSave")
        self.gridLayout_2.addWidget(self.btnFZSave, 2, 2, 1, 1)
        self.verticalLayout_2.addWidget(self.splitter)
        self.scrollArea.setWidget(self.scrollAreaWidgetContents)
        self.verticalLayout_3.addWidget(self.scrollArea)

        self.retranslateUi(GeneratorTab)
        self.tabWidget.setCurrentIndex(0)
        self.stackedWidgetFuzzing.setCurrentIndex(0)

    def retranslateUi(self, GeneratorTab):
        _translate = QtCore.QCoreApplication.translate
        GeneratorTab.setWindowTitle(_translate("GeneratorTab", "Form"))
        self.treeProtocols.setToolTip(
            _translate(
                "GeneratorTab",
                "<html><head/><body><p>Drag&amp;Drop Protocols to the table on the right to fill the generation table.</p></body></html>",
            )
        )
        self.tabWidget.setTabText(
            self.tabWidget.indexOf(self.tab_proto),
            _translate("GeneratorTab", "Protocols"),
        )
        self.lWPauses.setToolTip(
            _translate(
                "GeneratorTab",
                '<html><head/><body><p>The pauses will be added automatically when you drag a protocol from the tree above to the table on the right.<br/></p><p>You can see the <span style=" font-weight:600;">position</span> of each pause by <span style=" font-weight:600;">selecting it</span>. There will be drawn a line in the table indicating the position of the pause.<br/></p><p>Use context menu or double click to <span style=" font-weight:600;">edit a pauses\' length</span>.</p></body></html>',
            )
        )
        self.tabWidget.setTabText(
            self.tabWidget.indexOf(self.tab_pauses),
            _translate("GeneratorTab", "Pauses"),
        )
        self.groupBox.setTitle(
            _translate("GeneratorTab", "Add fuzzing values to generated data")
        )
        self.btnFuzz.setText(_translate("GeneratorTab", "Fuzz"))
        self.rBSuccessive.setToolTip(
            _translate(
                "GeneratorTab",
                '<html><head/><body><p>For multiple labels per message the fuzzed values are inserted <span style=" font-weight:600;">one-by-one</span>.</p></body></html>',
            )
        )
        self.rBSuccessive.setText(_translate("GeneratorTab", "S&uccessive"))
        self.rbConcurrent.setToolTip(
            _translate(
                "GeneratorTab",
                '<html><head/><body><p>For multiple labels per message the labels are fuzzed <span style=" font-weight:600;">at the same time</span>.</p></body></html>',
            )
        )
        self.rbConcurrent.setText(_translate("GeneratorTab", "&Concurrent"))
        self.rBExhaustive.setToolTip(
            _translate(
                "GeneratorTab",
                '<html><head/><body><p>For multiple labels per message the fuzzed values are inserted in <span style=" font-weight:600;">all possible combinations</span>.</p></body></html>',
            )
        )
        self.rBExhaustive.setText(_translate("GeneratorTab", "E&xhaustive"))
        self.progressBarFuzzing.setFormat(_translate("GeneratorTab", "%v/%m"))
        self.tabWidget.setTabText(
            self.tabWidget.indexOf(self.tab_fuzzing),
            _translate("GeneratorTab", "Fuzzing"),
        )
        self.lCarrierFreqValue.setText(_translate("GeneratorTab", "TextLabel"))
        self.lModType.setText(_translate("GeneratorTab", "Modulation Type:"))
        self.lModTypeValue.setText(_translate("GeneratorTab", "TextLabel"))
        self.label_carrier_phase.setText(_translate("GeneratorTab", "Carrier Phase:"))
        self.lCarrierPhaseValue.setText(_translate("GeneratorTab", "TextLabel"))
        self.lBitLength.setText(_translate("GeneratorTab", "Symbol Length:"))
        self.lBitLenValue.setText(_translate("GeneratorTab", "TextLabel"))
        self.lEncoding.setText(_translate("GeneratorTab", "Encoding:"))
        self.lEncodingValue.setText(_translate("GeneratorTab", "-"))
        self.lSampleRate.setText(_translate("GeneratorTab", "Sample Rate:"))
        self.lSampleRateValue.setText(_translate("GeneratorTab", "TextLabel"))
        self.lCarrierFrequency.setText(_translate("GeneratorTab", "Carrier Frequency:"))
        self.labelParameterValues.setText(_translate("GeneratorTab", "0/100"))
        self.lParamCaption.setText(_translate("GeneratorTab", "Amplitudes:"))
        self.label.setText(_translate("GeneratorTab", "Bits per Symbol:"))
        self.labelBitsPerSymbol.setText(_translate("GeneratorTab", "TextLabel"))
        self.cBoxModulations.setItemText(0, _translate("GeneratorTab", "MyModulation"))
        self.prBarGeneration.setFormat(_translate("GeneratorTab", "Modulating %p%"))
        self.btnSend.setText(_translate("GeneratorTab", "Send data..."))
        self.btnEditModulation.setText(_translate("GeneratorTab", "Edit ..."))
        self.lModulation.setText(_translate("GeneratorTab", "Modulation:"))
        self.btnGenerate.setToolTip(
            _translate(
                "GeneratorTab",
                "Generate the complex file of the modulated signal, after tuning all parameters above.",
            )
        )
        self.btnGenerate.setText(_translate("GeneratorTab", "Generate file..."))
        self.btnNetworkSDRSend.setToolTip(
            _translate(
                "GeneratorTab",
                '<html><head/><body><p><span style=" font-weight:600;">Send encoded data to your external application via TCP.</span></p></body></html>',
            )
        )
        self.btnNetworkSDRSend.setText(_translate("GeneratorTab", "Send via Network"))
        self.cbViewType.setItemText(0, _translate("GeneratorTab", "Bit"))
        self.cbViewType.setItemText(1, _translate("GeneratorTab", "Hex"))
        self.cbViewType.setItemText(2, _translate("GeneratorTab", "ASCII"))
        self.lViewType.setText(_translate("GeneratorTab", "Viewtype:"))
        self.btnRfCatSend.setToolTip(
            _translate(
                "GeneratorTab",
                '<html><head/><body><p><span style=" font-weight:600;">Send encoded data via RfCat. </span></p><p><span style=" font-style:italic;">Hit again for stopping the sending process. Note that you can set the number of repetitions (from 1 to infinite) in:</span></p><p><span style=" font-style:italic;">Edit-&gt;Options-&gt;Device-&gt;\'Device sending repetitions\'</span></p></body></html>',
            )
        )
        self.btnRfCatSend.setText(_translate("GeneratorTab", "Send via RfCat"))
        self.labelGeneratedData.setText(_translate("GeneratorTab", "Generated Data"))
        self.btnSave.setToolTip(
            _translate("GeneratorTab", "Save current fuzz profile.")
        )
        self.btnSave.setText(_translate("GeneratorTab", "..."))
        self.btnOpen.setToolTip(_translate("GeneratorTab", "Load a fuzz profile."))
        self.btnOpen.setText(_translate("GeneratorTab", "..."))
        self.lEstimatedTime.setToolTip(
            _translate(
                "GeneratorTab",
                "<html><head/><body><p>The estimated average time is based on the average number of bits per message and average sample rate, you set for the modulations.</p></body></html>",
            )
        )
        self.lEstimatedTime.setText(_translate("GeneratorTab", "Estimated Time: "))
        self.btnFZSave.setText(_translate("GeneratorTab", "Save as FlipperZero SubGHz"))


from urh.ui.ElidedLabel import ElidedLabel
from urh.ui.GeneratorListWidget import GeneratorListWidget
from urh.ui.views.GeneratorListView import GeneratorListView
from urh.ui.views.GeneratorTableView import GeneratorTableView
from urh.ui.views.GeneratorTreeView import GeneratorTreeView
from . import urh_rc



from collections import OrderedDict

import numpy as np

from multiprocessing.connection import Connection
from urh.dev.native.Device import Device

from urh.dev.native.lib import rtlsdr
from urh.util.Logger import logger


class RTLSDR(Device):
    DEVICE_LIB = rtlsdr
    ASYNCHRONOUS = False
    DEVICE_METHODS = Device.DEVICE_METHODS.copy()
    DEVICE_METHODS.update(
        {
            Device.Command.SET_RF_GAIN.name: "set_tuner_gain",
            Device.Command.SET_RF_GAIN.name + "_get_allowed_values": "get_tuner_gains",
            Device.Command.SET_BANDWIDTH.name: "set_tuner_bandwidth",
            Device.Command.SET_FREQUENCY_CORRECTION.name: "set_freq_correction",
            Device.Command.SET_DIRECT_SAMPLING_MODE.name: "set_direct_sampling",
            Device.Command.SET_BIAS_TEE_ENABLED.name: "set_bias_tee",
        }
    )

    DATA_TYPE = np.int8

    @classmethod
    def get_device_list(cls):
        return rtlsdr.get_device_list()

    @classmethod
    def setup_device(cls, ctrl_connection: Connection, device_identifier):
        # identifier gets set in self.receive_process_arguments
        device_number = int(device_identifier)
        ret = rtlsdr.open(device_number)
        ctrl_connection.send("OPEN (#{}):{}".format(device_number, ret))
        return ret == 0

    @classmethod
    def prepare_sync_receive(cls, ctrl_connection: Connection):
        ret = rtlsdr.reset_buffer()
        ctrl_connection.send("RESET_BUFFER:" + str(ret))
        return ret

    @classmethod
    def receive_sync(cls, data_conn: Connection):
        data_conn.send_bytes(rtlsdr.read_sync())

    @classmethod
    def shutdown_device(cls, ctrl_connection, is_tx: bool):
        logger.debug("RTLSDR: closing device")
        ret = rtlsdr.close()
        ctrl_connection.send("CLOSE:" + str(ret))

    def __init__(
        self, freq, gain, srate, device_number, resume_on_full_receive_buffer=False
    ):
        super().__init__(
            center_freq=freq,
            sample_rate=srate,
            bandwidth=0,
            gain=gain,
            if_gain=1,
            baseband_gain=1,
            resume_on_full_receive_buffer=resume_on_full_receive_buffer,
        )

        self.success = 0
        self.bandwidth_is_adjustable = (
            self.get_bandwidth_is_adjustable()
        )  # e.g. not in Manjaro Linux / Ubuntu 14.04

        self.device_number = device_number

        self.error_codes = {-100: "Method not available in installed driver."}

    @staticmethod
    def get_bandwidth_is_adjustable():
        return rtlsdr.bandwidth_is_adjustable()

    @property
    def device_parameters(self):
        return OrderedDict(
            [
                (self.Command.SET_FREQUENCY.name, self.frequency),
                (self.Command.SET_SAMPLE_RATE.name, self.sample_rate),
                (self.Command.SET_BANDWIDTH.name, self.bandwidth),
                (self.Command.SET_FREQUENCY_CORRECTION.name, self.freq_correction),
                (self.Command.SET_DIRECT_SAMPLING_MODE.name, self.direct_sampling_mode),
                (self.Command.SET_RF_GAIN.name, self.gain),
                (self.Command.SET_BIAS_TEE_ENABLED.name, self.bias_tee_enabled),
                ("identifier", self.device_number),
            ]
        )

    @property
    def has_multi_device_support(self):
        return True

    def set_device_bandwidth(self, bandwidth):
        if self.bandwidth_is_adjustable:
            super().set_device_bandwidth(bandwidth)
        else:
            logger.warning(
                "Setting the bandwidth is not supported by your RTL-SDR driver version."
            )

    @staticmethod
    def bytes_to_iq(buffer):
        return np.subtract(np.frombuffer(buffer, dtype=np.int8), 127).reshape(
            (-1, 2), order="C"
        )



from PyQt5.QtCore import QAbstractListModel, Qt, QModelIndex, pyqtSignal
from urh.signalprocessing.Participant import Participant


class ParticipantListModel(QAbstractListModel):
    show_state_changed = pyqtSignal()

    def __init__(self, participants, parent=None):
        """

        :type participants: list of Participant
        """
        super().__init__(parent)
        self.participants = participants
        self.show_unassigned = True

    def rowCount(self, QModelIndex_parent=None, *args, **kwargs):
        return len(self.participants) + 1

    def data(self, index: QModelIndex, role=None):
        row = index.row()
        if role == Qt.DisplayRole:
            if row == 0:
                return "not assigned"
            else:
                try:
                    return str(self.participants[row - 1])
                except IndexError:
                    return None

        elif role == Qt.CheckStateRole:
            if row == 0:
                return Qt.Checked if self.show_unassigned else Qt.Unchecked
            else:
                try:
                    return (
                        Qt.Checked if self.participants[row - 1].show else Qt.Unchecked
                    )
                except IndexError:
                    return None

    def setData(self, index: QModelIndex, value, role=None):
        if index.row() == 0 and role == Qt.CheckStateRole:
            if bool(value) != self.show_unassigned:
                self.show_unassigned = bool(value)
                self.show_state_changed.emit()

        elif role == Qt.CheckStateRole:
            try:
                if self.participants[index.row() - 1].show != value:
                    self.participants[index.row() - 1].show = value
                    self.show_state_changed.emit()
            except IndexError:
                return False

        return True

    def flags(self, index):
        return Qt.ItemIsEnabled | Qt.ItemIsSelectable | Qt.ItemIsUserCheckable

    def update(self):
        self.beginResetModel()
        self.endResetModel()



import os
import sys
import tempfile
from subprocess import call

build_dir = os.path.join(tempfile.gettempdir(), "build")


def main():
    cur_dir = os.path.realpath(__file__)
    os.chdir(os.path.realpath(os.path.join(cur_dir, "..", "..", "..", "..")))
    # call([sys.executable, "setup.py", "clean", "--all"])
    rc = call(
        [
            sys.executable,
            "setup.py",
            "build_ext",
            "--inplace",
            "-j{}".format(os.cpu_count()),
        ]
    )
    return rc


if __name__ == "__main__":
    sys.exit(main())



# -*- coding: utf-8 -*-

#
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_DialogFilterBandwidth(object):
    def setupUi(self, DialogFilterBandwidth):
        DialogFilterBandwidth.setObjectName("DialogFilterBandwidth")
        DialogFilterBandwidth.resize(502, 426)
        self.verticalLayout = QtWidgets.QVBoxLayout(DialogFilterBandwidth)
        self.verticalLayout.setSpacing(7)
        self.verticalLayout.setObjectName("verticalLayout")
        self.labelExplanation = QtWidgets.QLabel(DialogFilterBandwidth)
        sizePolicy = QtWidgets.QSizePolicy(
            QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred
        )
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(
            self.labelExplanation.sizePolicy().hasHeightForWidth()
        )
        self.labelExplanation.setSizePolicy(sizePolicy)
        self.labelExplanation.setAlignment(QtCore.Qt.AlignJustify | QtCore.Qt.AlignTop)
        self.labelExplanation.setWordWrap(True)
        self.labelExplanation.setObjectName("labelExplanation")
        self.verticalLayout.addWidget(self.labelExplanation)
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.doubleSpinBoxCustomBandwidth = QtWidgets.QDoubleSpinBox(
            DialogFilterBandwidth
        )
        sizePolicy = QtWidgets.QSizePolicy(
            QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed
        )
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(
            self.doubleSpinBoxCustomBandwidth.sizePolicy().hasHeightForWidth()
        )
        self.doubleSpinBoxCustomBandwidth.setSizePolicy(sizePolicy)
        self.doubleSpinBoxCustomBandwidth.setDecimals(4)
        self.doubleSpinBoxCustomBandwidth.setMinimum(0.0001)
        self.doubleSpinBoxCustomBandwidth.setMaximum(0.5)
        self.doubleSpinBoxCustomBandwidth.setObjectName("doubleSpinBoxCustomBandwidth")
        self.gridLayout.addWidget(self.doubleSpinBoxCustomBandwidth, 7, 1, 1, 1)
        self.spinBoxCustomKernelLength = QtWidgets.QSpinBox(DialogFilterBandwidth)
        self.spinBoxCustomKernelLength.setMinimum(8)
        self.spinBoxCustomKernelLength.setMaximum(999999999)
        self.spinBoxCustomKernelLength.setObjectName("spinBoxCustomKernelLength")
        self.gridLayout.addWidget(self.spinBoxCustomKernelLength, 7, 2, 1, 1)
        self.radioButtonCustom = QtWidgets.QRadioButton(DialogFilterBandwidth)
        self.radioButtonCustom.setObjectName("radioButtonCustom")
        self.gridLayout.addWidget(self.radioButtonCustom, 7, 0, 1, 1)
        self.labelMediumKernelLength = QtWidgets.QLabel(DialogFilterBandwidth)
        self.labelMediumKernelLength.setObjectName("labelMediumKernelLength")
        self.gridLayout.addWidget(self.labelMediumKernelLength, 4, 2, 1, 1)
        self.radioButtonWide = QtWidgets.QRadioButton(DialogFilterBandwidth)
        self.radioButtonWide.setObjectName("radioButtonWide")
        self.gridLayout.addWidget(self.radioButtonWide, 5, 0, 1, 1)
        self.labelVeryWideKernelLength = QtWidgets.QLabel(DialogFilterBandwidth)
        self.labelVeryWideKernelLength.setObjectName("labelVeryWideKernelLength")
        self.gridLayout.addWidget(self.labelVeryWideKernelLength, 6, 2, 1, 1)
        self.radioButtonNarrow = QtWidgets.QRadioButton(DialogFilterBandwidth)
        self.radioButtonNarrow.setObjectName("radioButtonNarrow")
        self.gridLayout.addWidget(self.radioButtonNarrow, 3, 0, 1, 1)
        self.labelWideBandwidth = QtWidgets.QLabel(DialogFilterBandwidth)
        self.labelWideBandwidth.setObjectName("labelWideBandwidth")
        self.gridLayout.addWidget(self.labelWideBandwidth, 5, 1, 1, 1)
        self.labelVeryNarrowKernelLength = QtWidgets.QLabel(DialogFilterBandwidth)
        self.labelVeryNarrowKernelLength.setObjectName("labelVeryNarrowKernelLength")
        self.gridLayout.addWidget(self.labelVeryNarrowKernelLength, 2, 2, 1, 1)
        self.labelBandwidthCaption = QtWidgets.QLabel(DialogFilterBandwidth)
        self.labelBandwidthCaption.setObjectName("labelBandwidthCaption")
        self.gridLayout.addWidget(self.labelBandwidthCaption, 0, 1, 1, 1)
        self.labelNarrowBandwidth = QtWidgets.QLabel(DialogFilterBandwidth)
        self.labelNarrowBandwidth.setObjectName("labelNarrowBandwidth")
        self.gridLayout.addWidget(self.labelNarrowBandwidth, 3, 1, 1, 1)
        self.labelNarrowKernelLength = QtWidgets.QLabel(DialogFilterBandwidth)
        self.labelNarrowKernelLength.setObjectName("labelNarrowKernelLength")
        self.gridLayout.addWidget(self.labelNarrowKernelLength, 3, 2, 1, 1)
        self.labelVeryNarrowBandwidth = QtWidgets.QLabel(DialogFilterBandwidth)
        self.labelVeryNarrowBandwidth.setObjectName("labelVeryNarrowBandwidth")
        self.gridLayout.addWidget(self.labelVeryNarrowBandwidth, 2, 1, 1, 1)
        self.labelKernelLengthCaption = QtWidgets.QLabel(DialogFilterBandwidth)
        self.labelKernelLengthCaption.setObjectName("labelKernelLengthCaption")
        self.gridLayout.addWidget(self.labelKernelLengthCaption, 0, 2, 1, 1)
        self.labelWideKernelLength = QtWidgets.QLabel(DialogFilterBandwidth)
        self.labelWideKernelLength.setObjectName("labelWideKernelLength")
        self.gridLayout.addWidget(self.labelWideKernelLength, 5, 2, 1, 1)
        self.radioButtonVery_Wide = QtWidgets.QRadioButton(DialogFilterBandwidth)
        self.radioButtonVery_Wide.setObjectName("radioButtonVery_Wide")
        self.gridLayout.addWidget(self.radioButtonVery_Wide, 6, 0, 1, 1)
        self.radioButtonVery_Narrow = QtWidgets.QRadioButton(DialogFilterBandwidth)
        self.radioButtonVery_Narrow.setObjectName("radioButtonVery_Narrow")
        self.gridLayout.addWidget(self.radioButtonVery_Narrow, 2, 0, 1, 1)
        self.label = QtWidgets.QLabel(DialogFilterBandwidth)
        sizePolicy = QtWidgets.QSizePolicy(
            QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Maximum
        )
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label.sizePolicy().hasHeightForWidth())
        self.label.setSizePolicy(sizePolicy)
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 0, 0, 1, 1)
        self.labelVeryWideBandwidth = QtWidgets.QLabel(DialogFilterBandwidth)
        self.labelVeryWideBandwidth.setObjectName("labelVeryWideBandwidth")
        self.gridLayout.addWidget(self.labelVeryWideBandwidth, 6, 1, 1, 1)
        self.radioButtonMedium = QtWidgets.QRadioButton(DialogFilterBandwidth)
        self.radioButtonMedium.setObjectName("radioButtonMedium")
        self.gridLayout.addWidget(self.radioButtonMedium, 4, 0, 1, 1)
        self.labelMediumBandwidth = QtWidgets.QLabel(DialogFilterBandwidth)
        self.labelMediumBandwidth.setObjectName("labelMediumBandwidth")
        self.gridLayout.addWidget(self.labelMediumBandwidth, 4, 1, 1, 1)
        self.line = QtWidgets.QFrame(DialogFilterBandwidth)
        self.line.setFrameShape(QtWidgets.QFrame.HLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.gridLayout.addWidget(self.line, 1, 0, 1, 3)
        self.verticalLayout.addLayout(self.gridLayout)
        self.buttonBox = QtWidgets.QDialogButtonBox(DialogFilterBandwidth)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(
            QtWidgets.QDialogButtonBox.Cancel | QtWidgets.QDialogButtonBox.Ok
        )
        self.buttonBox.setCenterButtons(True)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout.addWidget(self.buttonBox)
        self.verticalLayout.setStretch(0, 1)
        self.verticalLayout.setStretch(1, 3)
        self.verticalLayout.setStretch(2, 1)

        self.retranslateUi(DialogFilterBandwidth)
        self.buttonBox.accepted.connect(DialogFilterBandwidth.accept)
        self.buttonBox.rejected.connect(DialogFilterBandwidth.reject)

    def retranslateUi(self, DialogFilterBandwidth):
        _translate = QtCore.QCoreApplication.translate
        DialogFilterBandwidth.setWindowTitle(
            _translate("DialogFilterBandwidth", "Configure Filter Bandwidth")
        )
        self.labelExplanation.setText(
            _translate(
                "DialogFilterBandwidth",
                "<html><head/><body>To separate the frequency bands from each other a <b>bandpass</b> filter is used. You can configure the <b>bandwidth</b> of this filter here. The bandwidth determines the <b>length N</b> of the<b> filter kernel</b>. Decreasing the bandwidth will increase the accuracy of the filter, at cost of higher computation time.</body></html>",
            )
        )
        self.radioButtonCustom.setText(_translate("DialogFilterBandwidth", "Custom"))
        self.labelMediumKernelLength.setText(
            _translate("DialogFilterBandwidth", "TextLabel")
        )
        self.radioButtonWide.setText(_translate("DialogFilterBandwidth", "Wide"))
        self.labelVeryWideKernelLength.setText(
            _translate("DialogFilterBandwidth", "TextLabel")
        )
        self.radioButtonNarrow.setText(_translate("DialogFilterBandwidth", "Narrow"))
        self.labelWideBandwidth.setText(
            _translate("DialogFilterBandwidth", "TextLabel")
        )
        self.labelVeryNarrowKernelLength.setText(
            _translate("DialogFilterBandwidth", "TextLabel")
        )
        self.labelBandwidthCaption.setText(
            _translate("DialogFilterBandwidth", "<b>Bandwidth (Hz)</b>")
        )
        self.labelNarrowBandwidth.setText(
            _translate("DialogFilterBandwidth", "TextLabel")
        )
        self.labelNarrowKernelLength.setText(
            _translate("DialogFilterBandwidth", "TextLabel")
        )
        self.labelVeryNarrowBandwidth.setText(
            _translate("DialogFilterBandwidth", "TextLabel")
        )
        self.labelKernelLengthCaption.setText(
            _translate("DialogFilterBandwidth", "<b >Kernel Length N</b>")
        )
        self.labelWideKernelLength.setText(
            _translate("DialogFilterBandwidth", "TextLabel")
        )
        self.radioButtonVery_Wide.setText(
            _translate("DialogFilterBandwidth", "Very Wide")
        )
        self.radioButtonVery_Narrow.setText(
            _translate("DialogFilterBandwidth", "Very Narrow")
        )
        self.label.setText(_translate("DialogFilterBandwidth", "<b>Choose </b>"))
        self.labelVeryWideBandwidth.setText(
            _translate("DialogFilterBandwidth", "TextLabel")
        )
        self.radioButtonMedium.setText(_translate("DialogFilterBandwidth", "Medium"))
        self.labelMediumBandwidth.setText(
            _translate("DialogFilterBandwidth", "TextLabel")
        )



import array
import math
from collections import defaultdict

from PyQt5.QtCore import QAbstractTableModel, QModelIndex, Qt, pyqtSignal
from PyQt5.QtGui import QFont, QColor
from PyQt5.QtWidgets import QUndoStack

from urh import settings
from urh.signalprocessing.ChecksumLabel import ChecksumLabel
from urh.signalprocessing.ProtocolAnalyzer import ProtocolAnalyzer
from urh.ui.actions.InsertColumn import InsertColumn
from urh.util import util
from urh.util.Logger import logger


class TableModel(QAbstractTableModel):
    ALIGNMENT_CHAR = " "

    data_edited = pyqtSignal(int, int)
    vertical_header_color_status_changed = pyqtSignal(bool)

    def __init__(self, participants, parent=None):
        super().__init__(parent)
        self.controller = None  # :type: CompareFrameController|GeneratorTabController
        self.protocol = None  # type: ProtocolAnalyzer

        self.col_count = 0
        self.row_count = 0
        self.display_data = None  # type: list[str]

        self.search_results = []
        self.search_value = ""
        self._proto_view = 0
        self._refindex = -1

        self.first_messages = []
        self.hidden_rows = set()

        self.is_writeable = False
        self.locked = False
        self.decode = True  # False for Generator

        self.background_colors = defaultdict(lambda: None)
        self.bold_fonts = defaultdict(lambda: False)
        self.italic_fonts = defaultdict(lambda: False)
        self.text_colors = defaultdict(lambda: None)
        self.vertical_header_text = defaultdict(lambda: None)
        self.vertical_header_colors = defaultdict(lambda: None)

        self._diffs = defaultdict(set)  # type: dict[int, set[int]]

        self.undo_stack = QUndoStack()

        self.__participants = participants

    @property
    def participants(self):
        return self.__participants

    @participants.setter
    def participants(self, value):
        self.__participants = value
        for msg in self.protocol.messages:
            if msg.participant not in self.__participants:
                msg.participant = None

    @property
    def proto_view(self):
        return self._proto_view

    @proto_view.setter
    def proto_view(self, value):
        self._proto_view = value
        if self._refindex >= 0:
            self._diffs = self.find_differences(self._refindex)
        self.update()

    def get_alignment_offset_at(self, index: int):
        f = 1 if self.proto_view == 0 else 4 if self.proto_view == 1 else 8
        alignment_offset = int(
            math.ceil(self.protocol.messages[index].alignment_offset / f)
        )
        return alignment_offset

    def __pad_until_index(self, row: int, bit_pos: int):
        """
        Pad message in given row with zeros until given column so user can enter values behind end of message
        :return:
        """
        try:
            new_bits = array.array(
                "B", [0] * max(0, bit_pos - len(self.protocol.messages[row]))
            )
            if len(new_bits) == 0:
                return True

            self.protocol.messages[row].plain_bits = (
                self.protocol.messages[row].plain_bits + new_bits
            )
            msg = self.protocol.messages[row]
            self.display_data[row] = (
                msg.plain_bits
                if self.proto_view == 0
                else msg.plain_hex_array
                if self.proto_view == 1
                else msg.plain_ascii_array
            )
        except IndexError:
            return False

        return True

    def headerData(self, section: int, orientation, role=Qt.DisplayRole):
        if orientation == Qt.Vertical:
            if role == Qt.DisplayRole:
                return self.vertical_header_text[section]
            elif role == Qt.BackgroundColorRole:
                return self.vertical_header_colors[section]
            elif role == Qt.TextColorRole:
                color = self.vertical_header_colors[section]
                if color:
                    red, green, blue = color.red(), color.green(), color.blue()
                    return (
                        QColor("black")
                        if (red * 0.299 + green * 0.587 + blue * 0.114) > 186
                        else QColor("white")
                    )
                else:
                    return None

        return super().headerData(section, orientation, role)

    def update(self):
        self.locked = True

        if self.protocol.num_messages > 0:
            if self.decode:
                if self.proto_view == 0:
                    self.display_data = [
                        msg.decoded_bits for msg in self.protocol.messages
                    ]
                elif self.proto_view == 1:
                    self.display_data = [
                        msg.decoded_hex_array for msg in self.protocol.messages
                    ]
                elif self.proto_view == 2:
                    self.display_data = [
                        msg.decoded_ascii_array for msg in self.protocol.messages
                    ]
            else:
                # Generator Model
                if self.proto_view == 0:
                    self.display_data = [
                        msg.plain_bits for msg in self.protocol.messages
                    ]
                elif self.proto_view == 1:
                    self.display_data = [
                        msg.plain_hex_array for msg in self.protocol.messages
                    ]
                else:
                    self.display_data = [
                        msg.plain_ascii_array for msg in self.protocol.messages
                    ]

            visible_messages = [
                msg
                for i, msg in enumerate(self.display_data)
                if i not in self.hidden_rows
            ]
            if len(visible_messages) == 0:
                self.col_count = 0
            else:
                self.col_count = max(
                    len(msg) + self.get_alignment_offset_at(i)
                    for i, msg in enumerate(self.display_data)
                    if i not in self.hidden_rows
                )

            if self._refindex >= 0:
                self._diffs = self.find_differences(self._refindex)
            else:
                self._diffs.clear()

            self.row_count = self.protocol.num_messages
            self.find_protocol_value(self.search_value)
        else:
            self.col_count = 0
            self.row_count = 0
            self.display_data = None

        # Cache background colors for performance
        self.refresh_bgcolors()
        self.refresh_fonts()  # Will be overridden
        self.refresh_vertical_header()

        self.beginResetModel()
        self.endResetModel()
        self.locked = False

    def insert_column(self, index: int, rows: list):
        if self.protocol is None or not self.is_writeable:
            return

        insert_action = InsertColumn(self.protocol, index, rows, self.proto_view)
        self.undo_stack.push(insert_action)

    def columnCount(self, QModelIndex_parent=None, *args, **kwargs):
        return self.col_count

    def rowCount(self, QModelIndex_parent=None, *args, **kwargs):
        return self.row_count

    def refresh_bgcolors(self):
        self.background_colors.clear()
        label_colors = settings.LABEL_COLORS

        for i, message in enumerate(self.protocol.messages):
            for lbl in message.message_type:
                bg_color = label_colors[lbl.color_index]
                a = self.get_alignment_offset_at(i)
                start, end = message.get_label_range(lbl, self.proto_view, self.decode)
                for j in range(start, end):
                    self.background_colors[i, j + a] = bg_color

    def refresh_fonts(self):
        """
        Will be overridden

        :return:
        """
        pass

    def refresh_vertical_header(self):
        self.vertical_header_colors.clear()
        self.vertical_header_text.clear()
        use_colors = False
        for i in range(self.row_count):
            try:
                participant = self.protocol.messages[i].participant
            except IndexError:
                participant = None
            if participant:
                self.vertical_header_text[i] = "{0} ({1})".format(
                    i + 1, participant.shortname
                )
                self.vertical_header_colors[i] = settings.PARTICIPANT_COLORS[
                    participant.color_index
                ]
                use_colors = True
            else:
                self.vertical_header_text[i] = str(i + 1)

        self.vertical_header_color_status_changed.emit(use_colors)

    def data(self, index: QModelIndex, role=Qt.DisplayRole):
        if not index.isValid():
            return None

        i = index.row()
        j = index.column()
        if role == Qt.DisplayRole and self.display_data:
            try:
                alignment_offset = self.get_alignment_offset_at(i)
                if j < alignment_offset:
                    return self.ALIGNMENT_CHAR

                if self.proto_view == 0:
                    return self.display_data[i][j - alignment_offset]
                elif self.proto_view == 1:
                    return "{0:x}".format(self.display_data[i][j - alignment_offset])
                elif self.proto_view == 2:
                    return chr(self.display_data[i][j - alignment_offset])
            except IndexError:
                return None

        elif role == Qt.TextAlignmentRole:
            if i in self.first_messages:
                return Qt.AlignHCenter + Qt.AlignBottom
            else:
                return Qt.AlignCenter

        elif role == Qt.BackgroundColorRole:
            return self.background_colors[i, j]

        elif role == Qt.FontRole:
            font = QFont()
            font.setBold(self.bold_fonts[i, j])
            font.setItalic(self.italic_fonts[i, j])
            return font

        elif role == Qt.TextColorRole:
            return self.text_colors[i, j]

        elif role == Qt.ToolTipRole:
            return self.get_tooltip(i, j)
        else:
            return None

    def get_tooltip(self, row: int, column: int) -> str:
        msg = self.protocol.messages[row]
        try:
            lbl = next(
                lbl
                for lbl in msg.message_type
                if column
                in range(*msg.get_label_range(lbl, self.proto_view, self.decode))
            )
        except StopIteration:
            return ""

        result = lbl.name
        if isinstance(lbl, ChecksumLabel):
            calculated_crc = lbl.calculate_checksum_for_message(
                msg, use_decoded_bits=self.decode
            )
            start, end = msg.get_label_range(lbl=lbl, view=0, decode=self.decode)
            bits = msg.decoded_bits if self.decode else msg.plain_bits
            color = "green" if bits[start:end] == calculated_crc else "red"
            expected = util.convert_bits_to_string(calculated_crc, self.proto_view)
            result += '<br><font color="{}">Expected <b>{}</b></font>'.format(
                color, expected
            )

        return result

    def setData(self, index: QModelIndex, value, role=Qt.DisplayRole):
        if role != Qt.EditRole:
            return True

        i = index.row()
        j = index.column()
        a = self.get_alignment_offset_at(i)
        j -= a
        hex_chars = (
            "0",
            "1",
            "2",
            "3",
            "4",
            "5",
            "6",
            "7",
            "8",
            "9",
            "a",
            "b",
            "c",
            "d",
            "e",
            "f",
        )

        if i >= len(self.protocol.messages):
            return False

        if (
            self.proto_view == 0
            and value in ("0", "1")
            and self.__pad_until_index(i, j + 1)
        ):
            self.protocol.messages[i][j] = bool(int(value))
            self.display_data[i][j] = int(value)
        elif (
            self.proto_view == 1
            and value in hex_chars
            and self.__pad_until_index(i, (j + 1) * 4)
        ):
            converted_j = self.protocol.convert_index(
                j, 1, 0, self.decode, message_indx=i
            )[0]
            bits = "{0:04b}".format(int(value, 16))
            for k in range(4):
                self.protocol.messages[i][converted_j + k] = bool(int(bits[k]))
            self.display_data[i][j] = int(value, 16)
        elif (
            self.proto_view == 2
            and len(value) == 1
            and self.__pad_until_index(i, (j + 1) * 8)
        ):
            converted_j = self.protocol.convert_index(
                j, 2, 0, self.decode, message_indx=i
            )[0]
            bits = "{0:08b}".format(ord(value))
            for k in range(8):
                self.protocol.messages[i][converted_j + k] = bool(int(bits[k]))
            self.display_data[i][j] = ord(value)
        else:
            return False

        self.data_edited.emit(i, j)
        return True

    def find_protocol_value(self, value):
        self.search_results.clear()
        if self.proto_view == 1:
            value = value.lower()

        self.search_value = value

        if len(value) == 0:
            return 0

        for i, message in enumerate(self.protocol.messages):
            if i in self.hidden_rows:
                continue

            data = message.view_to_string(self.proto_view, self.decode)
            j = data.find(value)
            while j != -1:
                self.search_results.append((i, j + self.get_alignment_offset_at(i)))
                j = data.find(value, j + 1)

        return len(self.search_results)

    def find_differences(self, refindex: int):
        """
        Search all differences between protocol messages regarding a reference message

        :param refindex: index of reference message
        :rtype: dict[int, set[int]]
        """
        differences = defaultdict(set)

        if refindex >= len(self.protocol.messages):
            return differences

        if self.proto_view == 0:
            proto = self.protocol.decoded_proto_bits_str
        elif self.proto_view == 1:
            proto = self.protocol.decoded_hex_str
        elif self.proto_view == 2:
            proto = self.protocol.decoded_ascii_str
        else:
            return differences

        ref_message = proto[refindex]
        ref_offset = self.get_alignment_offset_at(refindex)

        for i, message in enumerate(proto):
            if i == refindex:
                continue

            msg_offset = self.get_alignment_offset_at(i)
            short, long = sorted(
                [len(ref_message) + ref_offset, len(message) + msg_offset]
            )

            differences[i] = {
                j
                for j in range(max(msg_offset, ref_offset), long)
                if j >= short or message[j - msg_offset] != ref_message[j - ref_offset]
            }

        return differences

    def get_selected_label_index(self, row: int, column: int):
        if self.row_count == 0:
            return -1

        try:
            msg = self.protocol.messages[row]
        except IndexError:
            logger.warning("{} is out of range for generator protocol".format(row))
            return -1

        for i, lbl in enumerate(msg.message_type):
            if column in range(*msg.get_label_range(lbl, self.proto_view, False)):
                return i

        return -1



import os
import sys
import tempfile

if sys.version_info < (3, 4):
    print("You need at least Python 3.4 for this application!")
    if sys.version_info[0] < 3:
        print("try running with python3 {}".format(" ".join(sys.argv)))
    sys.exit(1)

try:
    from setuptools import setup, Extension
    from setuptools.command.build_ext import build_ext as _build_ext
except ImportError:
    print("Could not find setuptools")
    print("Try installing them with pip install setuptools")
    sys.exit(1)

from src.urh.dev.native import ExtensionHelper
from src.urh.dev.native.ExtensionHelper import COMPILER_DIRECTIVES
import src.urh.version as version

if sys.platform == "win32":
    OPEN_MP_FLAG = "/openmp"
    NO_NUMPY_WARNINGS_FLAG = ""
elif sys.platform == "darwin":
    OPEN_MP_FLAG = ""  # no OpenMP support in default Mac OSX compiler
    NO_NUMPY_WARNINGS_FLAG = "-Wno-#warnings"
else:
    OPEN_MP_FLAG = "-fopenmp"
    NO_NUMPY_WARNINGS_FLAG = "-Wno-cpp"

UI_SUBDIRS = ("actions", "delegates", "views")
PLUGINS = [
    path
    for path in os.listdir("src/urh/plugins")
    if os.path.isdir(os.path.join("src/urh/plugins", path))
]
URH_DIR = "urh"

IS_RELEASE = os.path.isfile(os.path.join(tempfile.gettempdir(), "urh_releasing"))

try:
    from Cython.Build import cythonize
except ImportError:
    print(
        "You need Cython to build URH's extensions!\n"
        "You can get it e.g. with python3 -m pip install cython.",
        file=sys.stderr,
    )
    sys.exit(1)


def set_builtin(name, value):
    if isinstance(__builtins__, dict):
        __builtins__[name] = value
    else:
        # to support https://github.com/pypa/build
        # see https://github.com/jopohl/urh/issues/1106
        setattr(__builtins__, name, value)


class build_ext(_build_ext):
    def finalize_options(self):
        print("Finalizing options")
        _build_ext.finalize_options(self)
        # Prevent numpy from thinking it is still in its setup process:
        set_builtin("__NUMPY_SETUP__", False)
        import numpy

        self.include_dirs.append(numpy.get_include())


def get_packages():
    packages = [URH_DIR]
    separator = os.path.normpath("/")
    for dirpath, dirnames, filenames in os.walk(os.path.join("./src/", URH_DIR)):
        package_path = os.path.relpath(
            dirpath, os.path.join("./src/", URH_DIR)
        ).replace(separator, ".")
        if len(package_path) > 1:
            packages.append(URH_DIR + "." + package_path)

    return packages


def get_package_data():
    package_data = {"urh.cythonext": ["*.pyx", "*.pxd"]}
    for plugin in PLUGINS:
        package_data["urh.plugins." + plugin] = ["*.ui", "*.txt"]

    package_data["urh.dev.native.lib"] = ["*.pyx", "*.pxd"]

    if IS_RELEASE and sys.platform == "win32":
        package_data["urh.dev.native.lib.shared"] = ["*.dll", "*.txt"]

    return package_data


def get_extensions():
    filenames = [
        os.path.splitext(f)[0]
        for f in os.listdir("src/urh/cythonext")
        if f.endswith(".pyx")
    ]
    extensions = [
        Extension(
            "urh.cythonext." + f,
            ["src/urh/cythonext/" + f + ".pyx"],
            extra_compile_args=[OPEN_MP_FLAG],
            extra_link_args=[OPEN_MP_FLAG],
            language="c++",
        )
        for f in filenames
    ]

    ExtensionHelper.USE_RELATIVE_PATHS = True
    (
        device_extensions,
        device_extras,
    ) = ExtensionHelper.get_device_extensions_and_extras()
    extensions += device_extensions

    if NO_NUMPY_WARNINGS_FLAG:
        for extension in extensions:
            extension.extra_compile_args.append(NO_NUMPY_WARNINGS_FLAG)

    extensions = cythonize(
        extensions,
        compiler_directives=COMPILER_DIRECTIVES,
        compile_time_env=device_extras,
    )
    return extensions


def read_long_description():
    try:
        with open("README.md") as f:
            text = f.read()
        return text
    except:
        return ""


install_requires = ["numpy", "psutil", "cython"]
if IS_RELEASE:
    install_requires.append("pyqt5")
else:
    try:
        import PyQt5
    except ImportError:
        install_requires.append("pyqt5")

if sys.version_info < (3, 4):
    install_requires.append("enum34")

setup(
    name="urh",
    version=version.VERSION,
    description="Universal Radio Hacker: investigate wireless protocols like a boss",
    long_description=read_long_description(),
    long_description_content_type="text/markdown",
    author="Johannes Pohl",
    author_email="Johannes.Pohl90@gmail.com",
    package_dir={"": "src"},
    package_data=get_package_data(),
    url="https://github.com/jopohl/urh",
    license="GNU General Public License (GPL)",
    download_url="https://github.com/jopohl/urh/tarball/v" + str(version.VERSION),
    install_requires=install_requires,
    setup_requires=["numpy"],
    packages=get_packages(),
    ext_modules=get_extensions(),
    cmdclass={"build_ext": build_ext},
    zip_safe=False,
    entry_points={
        "console_scripts": [
            "urh = urh.main:main",
            "urh_cli = urh.cli.urh_cli:main",
        ]
    },
)



import sys
from collections import OrderedDict

from PyQt5.QtCore import QModelIndex, pyqtSlot, QAbstractItemModel, Qt
from PyQt5.QtGui import QPainter, QStandardItem
from PyQt5.QtWidgets import (
    QItemDelegate,
    QStyleOptionViewItem,
    QStyle,
    QComboBox,
    QStyledItemDelegate,
    QWidget,
)


class SectionItemDelegate(QItemDelegate):
    def __init__(self, parent=None):
        super().__init__(parent)

    def paint(
        self, painter: QPainter, option: QStyleOptionViewItem, index: QModelIndex
    ):
        item_type = index.data(Qt.AccessibleDescriptionRole)
        if item_type == "parent":
            parent_option = option
            parent_option.state |= QStyle.State_Enabled
            super().paint(painter, parent_option, index)
        elif item_type == "child":
            child_option = option
            indent = option.fontMetrics.width(4 * " ")
            child_option.rect.adjust(indent, 0, 0, 0)
            child_option.textElideMode = Qt.ElideNone
            super().paint(painter, child_option, index)
        else:
            super().paint(painter, option, index)


class SectionComboBox(QComboBox):
    def __init__(self, parent=None):
        super().__init__(parent)

    def add_parent_item(self, text):
        item = QStandardItem(text)
        item.setFlags(item.flags() & ~(Qt.ItemIsEnabled | Qt.ItemIsSelectable))
        item.setData("parent", Qt.AccessibleDescriptionRole)

        font = item.font()
        font.setBold(True)
        item.setFont(font)

        self.model().appendRow(item)

    def add_child_item(self, text):
        item = QStandardItem(text)
        item.setData("child", Qt.AccessibleDescriptionRole)
        self.model().appendRow(item)


class SectionComboBoxDelegate(QStyledItemDelegate):
    def __init__(self, items: OrderedDict, parent=None):
        """

        :param items:
        :param parent:
        """
        super().__init__(parent)
        self.items = items

    def createEditor(
        self, parent: QWidget, option: QStyleOptionViewItem, index: QModelIndex
    ):
        editor = SectionComboBox(parent)
        editor.setItemDelegate(SectionItemDelegate(editor.itemDelegate().parent()))
        if sys.platform == "win32":
            # Ensure text entries are visible with windows combo boxes
            editor.setMinimumHeight(self.sizeHint(option, index).height() + 10)

        for title, items in self.items.items():
            editor.add_parent_item(title)
            for item in items:
                editor.add_child_item(item)
        editor.currentIndexChanged.connect(self.current_index_changed)
        return editor

    def setEditorData(self, editor: SectionComboBox, index: QModelIndex):
        editor.blockSignals(True)
        item = index.model().data(index)
        editor.setCurrentText(item)
        editor.blockSignals(False)

    def setModelData(
        self, editor: QWidget, model: QAbstractItemModel, index: QModelIndex
    ):
        model.setData(index, editor.currentText(), Qt.EditRole)

    def updateEditorGeometry(
        self, editor: QWidget, option: QStyleOptionViewItem, index: QModelIndex
    ):
        editor.setGeometry(option.rect)

    @pyqtSlot()
    def current_index_changed(self):
        self.commitData.emit(self.sender())



from collections import OrderedDict

from PyQt5.QtCore import pyqtSlot, pyqtSignal, Qt
from PyQt5.QtGui import QIcon, QContextMenuEvent, QKeySequence
from PyQt5.QtWidgets import QTableView, QMenu, QAction

from urh import settings
from urh.models.LabelValueTableModel import LabelValueTableModel
from urh.signalprocessing.ProtocoLabel import ProtocolLabel
from urh.ui.delegates.ComboBoxDelegate import ComboBoxDelegate
from urh.ui.delegates.SectionComboBoxDelegate import SectionComboBoxDelegate


class LabelValueTableView(QTableView):
    edit_label_action_triggered = pyqtSignal()
    configure_field_types_action_triggered = pyqtSignal()

    def __init__(self, parent=None):
        super().__init__(parent)
        self.setItemDelegateForColumn(
            1,
            ComboBoxDelegate(
                [""] * len(settings.LABEL_COLORS),
                colors=settings.LABEL_COLORS,
                parent=self,
            ),
        )
        self.setItemDelegateForColumn(
            2, ComboBoxDelegate(ProtocolLabel.DISPLAY_FORMATS, parent=self)
        )

        orders = OrderedDict(
            [
                (
                    "Big Endian (BE)",
                    [bo + "/BE" for bo in ProtocolLabel.DISPLAY_BIT_ORDERS],
                ),
                (
                    "Little Endian (LE)",
                    [bo + "/LE" for bo in ProtocolLabel.DISPLAY_BIT_ORDERS],
                ),
            ]
        )

        self.setItemDelegateForColumn(3, SectionComboBoxDelegate(orders, parent=self))

        self.del_rows_action = QAction("Delete selected labels", self)
        self.del_rows_action.setShortcut(QKeySequence.Delete)
        self.del_rows_action.setIcon(QIcon.fromTheme("edit-delete"))
        self.del_rows_action.setShortcutContext(Qt.WidgetWithChildrenShortcut)
        self.del_rows_action.triggered.connect(self.delete_rows)

        self.addAction(self.del_rows_action)

    @property
    def selected_min_max_row(self):
        selected = self.selectionModel().selection()
        if selected.isEmpty():
            return -1, -1
        min_row = min(rng.top() for rng in selected)
        max_row = max(rng.bottom() for rng in selected)
        return min_row, max_row

    def create_context_menu(self):
        menu = QMenu()
        min_row, max_row = self.selected_min_max_row
        if self.model().rowCount() > 0 and min_row > -1:
            edit_label_action = menu.addAction(self.tr("Edit..."))
            edit_label_action.setIcon(QIcon.fromTheme("configure"))
            edit_label_action.triggered.connect(self.on_edit_label_action_triggered)

            if len(self.model().controller.proto_analyzer.message_types) > 1:
                msg_type_menu = menu.addMenu("Copy to message type")
                for i, message_type in enumerate(
                    self.model().controller.proto_analyzer.message_types
                ):
                    if message_type != self.model().controller.active_message_type:
                        msg_type_action = msg_type_menu.addAction(message_type.name)
                        msg_type_action.setData(i)
                        msg_type_action.triggered.connect(
                            self.on_copy_to_msg_type_action_triggered
                        )

            menu.addAction(self.del_rows_action)
        menu.addSeparator()
        configure_field_types_action = menu.addAction("Configure field types...")
        configure_field_types_action.triggered.connect(
            self.configure_field_types_action_triggered.emit
        )

        return menu

    def contextMenuEvent(self, event: QContextMenuEvent):
        menu = self.create_context_menu()
        menu.exec_(self.mapToGlobal(event.pos()))

    def model(self) -> LabelValueTableModel:
        return super().model()

    def delete_rows(self):
        min_row, max_row = self.selected_min_max_row
        if min_row > -1:
            self.model().delete_labels_at(min_row, max_row)

    @pyqtSlot()
    def on_edit_label_action_triggered(self):
        self.edit_label_action_triggered.emit()

    @pyqtSlot()
    def on_copy_to_msg_type_action_triggered(self):
        min_row, max_row = self.selected_min_max_row
        self.model().add_labels_to_message_type(
            min_row, max_row, int(self.sender().data())
        )



from PyQt5.QtCore import QItemSelection, pyqtSlot
from PyQt5.QtCore import pyqtSignal, QItemSelectionModel, Qt
from PyQt5.QtGui import QContextMenuEvent, QDropEvent, QIcon
from PyQt5.QtWidgets import QTreeView, QAbstractItemView, QMenu

from urh.models.ProtocolTreeModel import ProtocolTreeModel


class ProtocolTreeView(QTreeView):
    create_new_group_clicked = pyqtSignal()
    selection_changed = pyqtSignal()
    files_dropped_on_group = pyqtSignal(list, int)
    close_wanted = pyqtSignal(list)

    def __init__(self, parent=None):
        super().__init__(parent)
        self.setSelectionMode(QAbstractItemView.ExtendedSelection)
        self.setDragEnabled(True)
        self.setAcceptDrops(True)
        self.setDropIndicatorShown(True)

        self.move_to_group_actions = {}
        self.context_menu_pos = None

    def model(self) -> ProtocolTreeModel:
        return super().model()

    def selectionModel(self) -> QItemSelectionModel:
        return super().selectionModel()

    def create_context_menu(self):
        menu = QMenu()
        new_group_action = menu.addAction(self.tr("Create a new group"))
        new_group_action.setIcon(QIcon.fromTheme("list-add"))
        new_group_action.triggered.connect(self.on_new_group_action_triggered)

        item = self.model().getItem(self.indexAt(self.context_menu_pos))
        selected_items = [
            self.model().getItem(index)
            for index in self.selectionModel().selectedIndexes()
        ]
        selected_protocols = [
            item.protocol for item in selected_items if not item.is_group
        ]
        self.move_to_group_actions.clear()

        if item.is_group:
            delete_group_action = menu.addAction(self.tr("Delete group"))
            delete_group_action.setIcon(QIcon.fromTheme("list-remove"))
            delete_group_action.triggered.connect(self.on_delete_group_action_triggered)
        elif item != self.model().rootItem:
            tree_items = self.model().protocol_tree_items
            other_groups = [i for i in tree_items.keys() if item not in tree_items[i]]

            if len(selected_protocols) > 0:
                menu.addSeparator()
                close_action = menu.addAction(self.tr("Close"))
                close_action.setIcon(QIcon.fromTheme("window-close"))
                close_action.triggered.connect(self.on_close_action_triggered)

            if len(other_groups) > 0:
                move_to_group_menu = menu.addMenu("Move to Group")
                for i in other_groups:
                    group_name = self.model().rootItem.child(i).data()
                    move_to_group_action = move_to_group_menu.addAction(group_name)
                    move_to_group_action.triggered.connect(
                        self.on_move_to_group_action_triggered
                    )
                    self.move_to_group_actions[move_to_group_action] = i

        if item != self.model().rootItem:
            menu.addSeparator()
            sort_group_elements_action = menu.addAction("Sort Group Elements")
            sort_group_elements_action.setIcon(QIcon.fromTheme("view-sort-ascending"))
            sort_group_elements_action.triggered.connect(
                self.on_sort_group_elements_action_triggered
            )

        return menu

    def contextMenuEvent(self, event: QContextMenuEvent):
        self.context_menu_pos = event.pos()
        menu = self.create_context_menu()
        menu.exec(self.mapToGlobal(event.pos()))
        self.context_menu_pos = None

    def selectionChanged(self, selection1: QItemSelection, selection2: QItemSelection):
        self.selection_changed.emit()
        super().selectionChanged(selection1, selection2)

    def dropEvent(self, event: QDropEvent):
        if len(event.mimeData().urls()) > 0:
            group_id = self.model().get_group_id_for_index(self.indexAt(event.pos()))
            self.files_dropped_on_group.emit(event.mimeData().urls(), group_id)
        else:
            super().dropEvent(event)

    @pyqtSlot()
    def on_new_group_action_triggered(self):
        self.model().addGroup()
        self.model().update()

    @pyqtSlot()
    def on_move_to_group_action_triggered(self):
        selected_items = [
            self.model().getItem(index)
            for index in self.selectionModel().selectedIndexes()
        ]
        i = self.move_to_group_actions[self.sender()]
        self.model().move_to_group(selected_items, i)

    @pyqtSlot()
    def on_close_action_triggered(self):
        selected_items = [
            self.model().getItem(index)
            for index in self.selectionModel().selectedIndexes()
        ]
        selected_protocols = [
            item.protocol for item in selected_items if not item.is_group
        ]
        self.close_wanted.emit(selected_protocols)

    @pyqtSlot()
    def on_delete_group_action_triggered(self):
        item = self.model().getItem(self.indexAt(self.context_menu_pos))
        self.model().delete_group(item)

    @pyqtSlot()
    def on_sort_group_elements_action_triggered(self):
        item = self.model().getItem(self.indexAt(self.context_menu_pos))
        if item.is_group:
            sortgroup_id = self.model().rootItem.index_of(item)
        else:
            sortgroup_id = self.model().rootItem.index_of(item.parent())
        self.model().sort_group(sortgroup_id)



import locale
import os
import time

from PyQt5.QtCore import Qt, pyqtSlot
from PyQt5.QtGui import QCloseEvent
from PyQt5.QtWidgets import QDialog

from urh import settings
from urh.ui.ui_signal_details import Ui_SignalDetails
from urh.util.Formatter import Formatter


class SignalDetailsDialog(QDialog):
    def __init__(self, signal, parent=None):
        super().__init__(parent)
        self.signal = signal
        self.ui = Ui_SignalDetails()
        self.ui.setupUi(self)
        self.setAttribute(Qt.WA_DeleteOnClose)
        self.setWindowFlags(Qt.Window)

        file = self.signal.filename

        self.ui.lblName.setText(self.signal.name)

        if os.path.isfile(file):
            self.ui.lblFile.setText(file)
            self.ui.lblFileSize.setText(
                locale.format_string("%.2fMB", os.path.getsize(file) / (1024**2))
            )
            self.ui.lFileCreated.setText(time.ctime(os.path.getctime(file)))
        else:
            self.ui.lblFile.setText(self.tr("signal file not found"))
            self.ui.lblFileSize.setText("-")
            self.ui.lFileCreated.setText("-")

        self.ui.lblSamplesTotal.setText(
            "{0:n}".format(self.signal.num_samples).replace(",", " ")
        )
        self.ui.dsb_sample_rate.setValue(self.signal.sample_rate)
        self.set_duration()

        self.ui.dsb_sample_rate.valueChanged.connect(
            self.on_dsb_sample_rate_value_changed
        )
        self.restoreGeometry(
            settings.read("{}/geometry".format(self.__class__.__name__), type=bytes)
        )

    def closeEvent(self, event: QCloseEvent):
        settings.write(
            "{}/geometry".format(self.__class__.__name__), self.saveGeometry()
        )
        super().closeEvent(event)

    @pyqtSlot(float)
    def on_dsb_sample_rate_value_changed(self, value: float):
        self.signal.sample_rate = value
        self.set_duration()

    def set_duration(self):
        dur = self.signal.num_samples / self.signal.sample_rate
        self.ui.lDuration.setText(Formatter.science_time(dur))



import itertools
import math
import os
import time
from collections import defaultdict

import numpy as np

from urh.cythonext import awre_util
from urh.signalprocessing.FieldType import FieldType


class Preprocessor(object):
    """
    This class preprocesses the messages in the following ways
    1) Identify preamble / length of preamble
    2) Identify sync word(s)
    3) Align all given messages on the identified preamble information
    """

    _DEBUG_ = False

    def __init__(self, bitvectors: list, existing_message_types: dict = None):
        self.bitvectors = bitvectors  # type: list[np.ndarray]
        self.existing_message_types = (
            existing_message_types if existing_message_types is not None else dict()
        )

    def preprocess(self) -> (np.ndarray, int):
        raw_preamble_positions = self.get_raw_preamble_positions()
        existing_sync_words = self.__get_existing_sync_words()
        if len(existing_sync_words) == 0:
            sync_words = self.find_possible_syncs(raw_preamble_positions)
        else:
            # NOTE: This does not cover the case if protocol has multiple sync words and not all of them were labeled
            sync_words = existing_sync_words

        preamble_starts = raw_preamble_positions[:, 0]
        preamble_lengths = self.get_preamble_lengths_from_sync_words(
            sync_words, preamble_starts=preamble_starts
        )
        sync_len = len(sync_words[0]) if len(sync_words) > 0 else 0
        return preamble_starts, preamble_lengths, sync_len

    def get_preamble_lengths_from_sync_words(
        self, sync_words: list, preamble_starts: np.ndarray
    ):
        """
        Get the preamble lengths based on the found sync words for all messages.
        If there should be more than one sync word in a message, use the first one.

        :param sync_words:
        :param preamble_starts:
        :return:
        """
        # If there should be varying sync word lengths we need to return an array of sync lengths per message
        assert all(len(sync_word) == len(sync_words[0]) for sync_word in sync_words)

        byte_sync_words = [bytes(map(int, sync_word)) for sync_word in sync_words]

        result = np.zeros(len(self.bitvectors), dtype=np.uint32)

        for i, bitvector in enumerate(self.bitvectors):
            preamble_lengths = []
            bits = bitvector.tobytes()

            for sync_word in byte_sync_words:
                sync_start = bits.find(sync_word)
                if sync_start != -1:
                    if sync_start - preamble_starts[i] >= 2:
                        preamble_lengths.append(sync_start - preamble_starts[i])

                    # Consider case where sync word starts with preamble pattern
                    sync_start = bits.find(
                        sync_word, sync_start + 1, sync_start + 2 * len(sync_word)
                    )

                    if sync_start != -1:
                        if sync_start - preamble_starts[i] >= 2:
                            preamble_lengths.append(sync_start - preamble_starts[i])

            preamble_lengths.sort()

            if len(preamble_lengths) == 0:
                result[i] = 0
            elif len(preamble_lengths) == 1:
                result[i] = preamble_lengths[0]
            else:
                # consider all indices not more than one byte before first one
                preamble_lengths = list(
                    filter(lambda x: x < preamble_lengths[0] + 7, preamble_lengths)
                )

                # take the smallest preamble_length, but prefer a greater one if it is divisible by 8 (or 4)
                preamble_length = next(
                    (pl for pl in preamble_lengths if pl % 8 == 0), None
                )
                if preamble_length is None:
                    preamble_length = next(
                        (pl for pl in preamble_lengths if pl % 4 == 0), None
                    )
                if preamble_length is None:
                    preamble_length = preamble_lengths[0]
                result[i] = preamble_length

        return result

    def find_possible_syncs(self, raw_preamble_positions=None):
        difference_matrix = self.get_difference_matrix()
        if raw_preamble_positions is None:
            raw_preamble_positions = self.get_raw_preamble_positions()
        return self.determine_sync_candidates(
            raw_preamble_positions, difference_matrix, n_gram_length=4
        )

    @staticmethod
    def merge_possible_sync_words(possible_sync_words: dict, n_gram_length: int):
        """
        Merge possible sync words by looking for common prefixes

        :param possible_sync_words: dict of possible sync words and their frequencies
        :return:
        """
        result = defaultdict(int)
        if len(possible_sync_words) < 2:
            return possible_sync_words.copy()

        for sync1, sync2 in itertools.combinations(possible_sync_words, 2):
            common_prefix = os.path.commonprefix([sync1, sync2])
            if len(common_prefix) > n_gram_length:
                result[common_prefix] += (
                    possible_sync_words[sync1] + possible_sync_words[sync2]
                )
            else:
                result[sync1] += possible_sync_words[sync1]
                result[sync2] += possible_sync_words[sync2]
        return result

    def determine_sync_candidates(
        self,
        raw_preamble_positions: np.ndarray,
        difference_matrix: np.ndarray,
        n_gram_length=4,
    ) -> list:
        possible_sync_words = awre_util.find_possible_sync_words(
            difference_matrix, raw_preamble_positions, self.bitvectors, n_gram_length
        )

        self.__debug("Possible sync words", possible_sync_words)
        if len(possible_sync_words) == 0:
            return []

        possible_sync_words = self.merge_possible_sync_words(
            possible_sync_words, n_gram_length
        )
        self.__debug("Merged sync words", possible_sync_words)

        scores = self.__score_sync_lengths(possible_sync_words)

        sorted_scores = sorted(scores, reverse=True, key=scores.get)
        estimated_sync_length = sorted_scores[0]
        if estimated_sync_length % 8 != 0:
            for other in filter(
                lambda x: 0 < estimated_sync_length - x < 7, sorted_scores
            ):
                if other % 8 == 0:
                    estimated_sync_length = other
                    break

        # Now we look at all possible sync words with this length
        sync_words = {
            word: frequency
            for word, frequency in possible_sync_words.items()
            if len(word) == estimated_sync_length
        }
        self.__debug("Sync words", sync_words)

        additional_syncs = self.__find_additional_sync_words(
            estimated_sync_length, sync_words, possible_sync_words
        )

        if additional_syncs:
            self.__debug("Found additional sync words", additional_syncs)
            sync_words.update(additional_syncs)

        result = []
        for sync_word in sorted(sync_words, key=sync_words.get, reverse=True):
            # Convert bytes back to string
            result.append("".join(str(c) for c in sync_word))

        return result

    def __find_additional_sync_words(
        self, sync_length: int, present_sync_words, possible_sync_words
    ) -> dict:
        """
        Look for additional sync words, in case we had varying preamble lengths and multiple sync words
        (see test_with_three_syncs_different_preamble_lengths for an example)

        :param sync_length:
        :type present_sync_words: dict
        :type possible_sync_words: dict
        :return:
        """
        np_syn = [
            np.fromiter(map(int, sync_word), dtype=np.uint8, count=len(sync_word))
            for sync_word in present_sync_words
        ]

        messages_without_sync = [
            i
            for i, bv in enumerate(self.bitvectors)
            if not any(
                awre_util.find_occurrences(bv, s, return_after_first=True)
                for s in np_syn
            )
        ]

        result = dict()
        if len(messages_without_sync) == 0:
            return result

        # Is there another sync word that applies to all messages without sync?
        additional_candidates = {
            word: score
            for word, score in possible_sync_words.items()
            if len(word) > sync_length
            and not any(s in word for s in present_sync_words)
        }

        for sync in sorted(
            additional_candidates, key=additional_candidates.get, reverse=True
        ):
            if len(messages_without_sync) == 0:
                break

            score = additional_candidates[sync]
            s = sync[:sync_length]
            np_s = np.fromiter(s, dtype=np.uint8, count=len(s))
            matching = [
                i
                for i in messages_without_sync
                if awre_util.find_occurrences(
                    self.bitvectors[i], np_s, return_after_first=True
                )
            ]
            if matching:
                result[s] = score
                for m in matching:
                    messages_without_sync.remove(m)

        return result

    def get_raw_preamble_positions(self) -> np.ndarray:
        """
        Return a 2D numpy array where first column is the start of preamble
        second and third columns are lower and upper bound for preamble length by message, respectively
        """
        result = np.zeros((len(self.bitvectors), 3), dtype=np.uint32)

        for i, bitvector in enumerate(self.bitvectors):
            if i in self.existing_message_types:
                preamble_label = self.existing_message_types[
                    i
                ].get_first_label_with_type(FieldType.Function.PREAMBLE)
            else:
                preamble_label = None

            if preamble_label is None:
                start, lower, upper = awre_util.get_raw_preamble_position(bitvector)
            else:
                # If this message is already labeled with a preamble we just use it's values
                start, lower, upper = (
                    preamble_label.start,
                    preamble_label.end,
                    preamble_label.end,
                )

            result[i, 0] = start
            result[i, 1] = lower - start
            result[i, 2] = upper - start

        return result

    def get_difference_matrix(self) -> np.ndarray:
        """
        Return a matrix of the first difference index between all messages
        :return:
        """
        return awre_util.get_difference_matrix(self.bitvectors)

    def __score_sync_lengths(self, possible_sync_words: dict):
        sync_lengths = defaultdict(int)
        for sync_word, score in possible_sync_words.items():
            sync_lengths[len(sync_word)] += score

        self.__debug("Sync lengths", sync_lengths)

        return sync_lengths

    def __get_existing_sync_words(self) -> list:
        result = []
        for i, bitvector in enumerate(self.bitvectors):
            if i in self.existing_message_types:
                sync_label = self.existing_message_types[i].get_first_label_with_type(
                    FieldType.Function.SYNC
                )
            else:
                sync_label = None

            if sync_label is not None:
                result.append(
                    "".join(map(str, bitvector[sync_label.start : sync_label.end]))
                )
        return result

    def __debug(self, *args):
        if self._DEBUG_:
            print("[PREPROCESSOR]", *args)

    @staticmethod
    def get_next_multiple_of_n(number: int, n: int):
        return n * int(math.ceil(number / n))

    @staticmethod
    def lower_multiple_of_n(number: int, n: int):
        return n * int(math.floor(number / n))

    @staticmethod
    def get_next_lower_multiple_of_two(number: int):
        return number if number % 2 == 0 else number - 1



#!/usr/bin/env python3
import argparse
import logging
import os
import sys
import time
from collections import defaultdict

import numpy as np

DEFAULT_CARRIER_FREQUENCY = 1e3
DEFAULT_CARRIER_AMPLITUDE = 1
DEFAULT_CARRIER_PHASE = 0

DEFAULT_SAMPLES_PER_SYMBOL = 100
DEFAULT_NOISE = 0.1
DEFAULT_CENTER = 0
DEFAULT_CENTER_SPACING = 0.1
DEFAULT_TOLERANCE = 5

cli_exe = sys.executable if hasattr(sys, "frozen") else sys.argv[0]
cur_dir = os.path.realpath(os.path.dirname(os.path.realpath(cli_exe)))
SRC_DIR = os.path.realpath(os.path.join(cur_dir, "..", ".."))

if os.path.isdir(SRC_DIR):
    sys.path.insert(0, SRC_DIR)

from urh.signalprocessing.IQArray import IQArray
from urh.util import util

util.set_shared_library_path()

try:
    import urh.cythonext.signal_functions
    import urh.cythonext.path_creator
    import urh.cythonext.util
except ImportError:
    if hasattr(sys, "frozen"):
        print("C++ Extensions not found. Exiting...")
        sys.exit(1)
    print("Could not find C++ extensions, trying to build them.")
    old_dir = os.path.realpath(os.curdir)
    os.chdir(os.path.join(SRC_DIR, "urh", "cythonext"))

    from urh.cythonext import build

    build.main()

    os.chdir(old_dir)

from urh.dev.BackendHandler import BackendHandler
from urh.signalprocessing.Modulator import Modulator
from urh.dev.VirtualDevice import VirtualDevice
from urh.signalprocessing.ProtocolSniffer import ProtocolSniffer
from urh.util import Logger
from urh.settings import PAUSE_SEP
from urh.util.Logger import logger
from urh.signalprocessing.Encoding import Encoding
from urh.signalprocessing.ProtocolAnalyzer import ProtocolAnalyzer

DEVICES = BackendHandler.DEVICE_NAMES
MODULATIONS = Modulator.MODULATION_TYPES


def cli_progress_bar(value, end_value, bar_length=20, title="Percent"):
    percent = value / end_value
    hashes = "#" * int(round(percent * bar_length))
    spaces = " " * (bar_length - len(hashes))
    sys.stdout.write(
        "\r{0}:\t[{1}] {2}%".format(title, hashes + spaces, int(round(percent * 100)))
    )
    sys.stdout.flush()


def on_fatal_device_error_occurred(error: str):
    logger.critical(error.strip())
    sys.exit(1)


def build_modulator_from_args(arguments: argparse.Namespace):
    if arguments.raw:
        return None
    if arguments.bits_per_symbol is None:
        arguments.bits_per_symbol = 1

    n = 2 ** int(arguments.bits_per_symbol)
    if arguments.parameters is None or len(arguments.parameters) != n:
        raise ValueError(
            "You need to give {} parameters for {} bits per symbol".format(
                n, int(arguments.bits_per_symbol)
            )
        )

    result = Modulator("CLI Modulator")
    result.carrier_freq_hz = float(arguments.carrier_frequency)
    result.carrier_amplitude = float(arguments.carrier_amplitude)
    result.carrier_phase_deg = float(arguments.carrier_phase)
    result.samples_per_symbol = int(arguments.samples_per_symbol)
    result.bits_per_symbol = int(arguments.bits_per_symbol)
    result.modulation_type = arguments.modulation_type
    result.sample_rate = arguments.sample_rate

    for i, param in enumerate(arguments.parameters):
        if result.is_amplitude_based and param.endswith("%"):
            result.parameters[i] = float(param[:-1])
        elif result.is_amplitude_based and not param.endswith("%"):
            result.parameters[i] = float(param) * 100
        else:
            result.parameters[i] = float(param)

    return result


def build_backend_handler_from_args(arguments: argparse.Namespace):
    from urh.dev.BackendHandler import Backends

    bh = BackendHandler()
    if arguments.device_backend == "native":
        bh.device_backends[arguments.device.lower()].selected_backend = Backends.native
    elif arguments.device_backend == "gnuradio":
        bh.device_backends[arguments.device.lower()].selected_backend = Backends.grc
    else:
        raise ValueError("Unsupported device backend")
    return bh


def build_device_from_args(arguments: argparse.Namespace):
    from urh.dev.VirtualDevice import Mode

    bh = build_backend_handler_from_args(arguments)

    bandwidth = (
        arguments.sample_rate if arguments.bandwidth is None else arguments.bandwidth
    )
    result = VirtualDevice(
        bh,
        name=arguments.device,
        mode=Mode.receive if arguments.receive else Mode.send,
        freq=arguments.frequency,
        sample_rate=arguments.sample_rate,
        bandwidth=bandwidth,
        gain=arguments.gain,
        if_gain=arguments.if_gain,
        baseband_gain=arguments.baseband_gain,
    )

    result.freq_correction = arguments.frequency_correction
    if arguments.device_identifier is not None:
        try:
            result.device_number = int(arguments.device_identifier)
        except ValueError:
            result.device_serial = arguments.device_identifier

    result.fatal_error_occurred.connect(on_fatal_device_error_occurred)

    return result


def build_protocol_sniffer_from_args(arguments: argparse.Namespace):
    bh = build_backend_handler_from_args(arguments)

    result = ProtocolSniffer(
        arguments.samples_per_symbol,
        arguments.center,
        arguments.center_spacing,
        arguments.noise,
        arguments.tolerance,
        arguments.modulation_type,
        arguments.bits_per_symbol,
        arguments.device.lower(),
        bh,
    )
    result.rcv_device.frequency = arguments.frequency
    result.rcv_device.sample_rate = arguments.sample_rate
    result.rcv_device.bandwidth = (
        arguments.sample_rate if arguments.bandwidth is None else arguments.bandwidth
    )
    result.rcv_device.freq_correction = arguments.frequency_correction
    if arguments.gain is not None:
        result.rcv_device.gain = arguments.gain
    if arguments.if_gain is not None:
        result.rcv_device.if_gain = arguments.if_gain
    if arguments.baseband_gain is not None:
        result.rcv_device.baseband_gain = arguments.baseband_gain

    if arguments.device_identifier is not None:
        try:
            result.rcv_device.device_number = int(arguments.device_identifier)
        except ValueError:
            result.rcv_device.device_serial = arguments.device_identifier

    if arguments.encoding:
        result.decoder = build_encoding_from_args(arguments)

    result.rcv_device.fatal_error_occurred.connect(on_fatal_device_error_occurred)
    result.adaptive_noise = arguments.adaptive_noise
    return result


def build_encoding_from_args(arguments: argparse.Namespace):
    if arguments.encoding is None:
        return None

    primitives = arguments.encoding.split(",")
    return Encoding(list(filter(None, map(str.strip, primitives))))


def read_messages_to_send(arguments: argparse.Namespace):
    if not arguments.transmit:
        return None

    if arguments.messages is not None and arguments.filename is not None:
        print("Either give messages (-m) or a file to read from (-file) not both.")
        sys.exit(1)
    elif arguments.messages is not None:
        # support for calls from external tools e.g. metasploit
        if len(arguments.messages) == 1:
            message_strings = arguments.messages[0].split(" ")
        else:
            message_strings = arguments.messages
    elif arguments.filename is not None:
        with open(arguments.filename) as f:
            message_strings = list(map(str.strip, f.readlines()))
    else:
        print(
            "You need to give messages to send either with (-m) or a file (-file) to read them from."
        )
        sys.exit(1)

    encoding = build_encoding_from_args(arguments)

    result = ProtocolAnalyzer.get_protocol_from_string(
        message_strings,
        is_hex=arguments.hex,
        default_pause=arguments.pause,
        sample_rate=arguments.sample_rate,
    ).messages
    if encoding:
        for msg in result:
            msg.decoder = encoding

    return result


def modulate_messages(messages, modulator):
    if len(messages) == 0:
        return None

    cli_progress_bar(0, len(messages), title="Modulating")
    nsamples = sum(
        int(len(msg.encoded_bits) * modulator.samples_per_symbol + msg.pause)
        for msg in messages
    )
    buffer = IQArray(None, dtype=np.float32, n=nsamples)
    pos = 0
    for i, msg in enumerate(messages):
        # We do not need to modulate the pause extra, as result is already initialized with zeros
        modulated = modulator.modulate(start=0, data=msg.encoded_bits, pause=0)
        buffer[pos : pos + len(modulated)] = modulated
        pos += len(modulated) + msg.pause
        cli_progress_bar(i + 1, len(messages), title="Modulating")
    print("\nSuccessfully modulated {} messages".format(len(messages)))
    return buffer


def parse_project_file(file_path: str):
    import xml.etree.ElementTree as ET
    from urh.util.ProjectManager import ProjectManager

    result = defaultdict(lambda: None)
    if not file_path or not os.path.isfile(file_path):
        return result

    try:
        tree = ET.parse(file_path)
        root = tree.getroot()
    except Exception as e:
        logger.error("Could not read project file {}: {}".format(file_path, e))
        return result

    ProjectManager.read_device_conf_dict(root.find("device_conf"), target_dict=result)
    result["device"] = result["name"]

    modulators = Modulator.modulators_from_xml_tag(root)
    if len(modulators) > 0:
        modulator = modulators[0]
        result["carrier_frequency"] = modulator.carrier_freq_hz
        result["carrier_amplitude"] = modulator.carrier_amplitude
        result["carrier_phase"] = modulator.carrier_phase_deg
        result["parameters"] = " ".join(map(str, modulator.parameters))
        result["modulation_type"] = modulator.modulation_type

    return result


def create_parser():
    parser = argparse.ArgumentParser(
        description="This is the Command Line Interface for the Universal Radio Hacker.",
        add_help=False,
    )
    parser.add_argument("project_file", nargs="?", default=None)

    group1 = parser.add_argument_group(
        "Software Defined Radio Settings", "Configure Software Defined Radio options"
    )
    group1.add_argument(
        "-d",
        "--device",
        choices=DEVICES,
        metavar="DEVICE",
        help="Choose a Software Defined Radio. Allowed values are "
        + ", ".join(DEVICES),
    )
    group1.add_argument("-di", "--device-identifier")
    group1.add_argument(
        "-db", "--device-backend", choices=["native", "gnuradio"], default="native"
    )
    group1.add_argument(
        "-f",
        "--frequency",
        type=float,
        help="Center frequency the SDR shall be tuned to",
    )
    group1.add_argument("-s", "--sample-rate", type=float, help="Sample rate to use")
    group1.add_argument(
        "-b",
        "--bandwidth",
        type=float,
        help="Bandwidth to use (defaults to sample rate)",
    )
    group1.add_argument("-g", "--gain", type=int, help="RF gain the SDR shall use")
    group1.add_argument(
        "-if",
        "--if-gain",
        type=int,
        help="IF gain to use (only supported for some SDRs)",
    )
    group1.add_argument(
        "-bb",
        "--baseband-gain",
        type=int,
        help="Baseband gain to use (only supported for some SDRs)",
    )
    group1.add_argument(
        "-a",
        "--adaptive-noise",
        action="store_true",
        help="Use adaptive noise when receiving.",
    )
    group1.add_argument(
        "-fcorr",
        "--frequency-correction",
        default=1,
        type=int,
        help="Set the frequency correction for SDR (if supported)",
    )

    group2 = parser.add_argument_group(
        "Modulation/Demodulation settings",
        "Configure the Modulator/Demodulator. Not required in raw mode."
        "In case of RX there are additional demodulation options.",
    )
    group2.add_argument(
        "-cf",
        "--carrier-frequency",
        type=float,
        help="Carrier frequency in Hertz (default: {})".format(
            DEFAULT_CARRIER_FREQUENCY
        ),
    )
    group2.add_argument(
        "-ca",
        "--carrier-amplitude",
        type=float,
        help="Carrier amplitude (default: {})".format(DEFAULT_CARRIER_AMPLITUDE),
    )
    group2.add_argument(
        "-cp",
        "--carrier-phase",
        type=float,
        help="Carrier phase in degree (default: {})".format(DEFAULT_CARRIER_PHASE),
    )
    group2.add_argument(
        "-mo",
        "--modulation-type",
        choices=MODULATIONS,
        metavar="MOD_TYPE",
        default="FSK",
        help="Modulation type must be one of "
        + ", ".join(MODULATIONS)
        + " (default: %(default)s)",
    )
    group2.add_argument(
        "-bps",
        "--bits-per-symbol",
        type=int,
        help="Bits per symbol e.g. 1 means binary modulation (default: 1).",
    )
    group2.add_argument(
        "-pm",
        "--parameters",
        nargs="+",
        help="Parameters for modulation. Separate with spaces",
    )

    # Legacy
    group2.add_argument("-p0", "--parameter-zero", help=argparse.SUPPRESS)
    group2.add_argument("-p1", "--parameter-one", help=argparse.SUPPRESS)

    group2.add_argument(
        "-sps",
        "--samples-per-symbol",
        type=int,
        help="Length of a symbol in samples (default: {}).".format(
            DEFAULT_SAMPLES_PER_SYMBOL
        ),
    )
    group2.add_argument(
        "-bl",
        "--bit-length",
        type=int,
        help="Same as samples per symbol, just there for legacy support (default: {}).".format(
            DEFAULT_SAMPLES_PER_SYMBOL
        ),
    )

    group2.add_argument(
        "-n",
        "--noise",
        type=float,
        help="Noise threshold (default: {}). Used for RX only.".format(DEFAULT_NOISE),
    )
    group2.add_argument(
        "-c",
        "--center",
        type=float,
        help="Center between symbols for demodulation (default: {}). "
        "Used for RX only.".format(DEFAULT_CENTER),
    )
    group2.add_argument(
        "-cs",
        "--center-spacing",
        type=float,
        help="Center spacing between symbols for demodulation (default: {}). "
        "Value has only effect for modulations with more than 1 bit per symbol. "
        "Used only for RX.".format(DEFAULT_CENTER_SPACING),
    )
    group2.add_argument(
        "-t",
        "--tolerance",
        type=float,
        help="Tolerance for demodulation in samples (default: {}). "
        "Used for RX only.".format(DEFAULT_TOLERANCE),
    )

    group3 = parser.add_argument_group(
        "Data configuration", "Configure which data to send or where to receive it."
    )
    group3.add_argument(
        "--hex", action="store_true", help="Give messages as hex instead of bits"
    )
    group3.add_argument("-e", "--encoding", help="Specify encoding")
    group3.add_argument(
        "-m",
        "--messages",
        nargs="+",
        help="Messages to send. Give pauses after with a {0}. "
        "Separate with spaces e.g. "
        "1001{0}42ms 1100{0}3ns 0001 1111{0}200. "
        "If you give no time suffix after a pause "
        "it is assumed to be in samples. ".format(PAUSE_SEP),
    )

    group3.add_argument(
        "-file",
        "--filename",
        help="Filename to read messages from in send mode. "
        "In receive mode messages will be written to this file "
        "instead to STDOUT.",
    )
    group3.add_argument(
        "-p",
        "--pause",
        default="250ms",
        help="The default pause which is inserted after a every message "
        "which does not have a pause configured. (default: %(default)s) "
        "Supported time units: s (second), ms (millisecond), µs (microsecond), ns (nanosecond) "
        "If you do not give a time suffix the pause is assumed to be in samples.",
    )
    group3.add_argument("-rx", "--receive", action="store_true", help="Enter RX mode")
    group3.add_argument("-tx", "--transmit", action="store_true", help="Enter TX mode")
    group3.add_argument(
        "-rt",
        "--receive-time",
        default="3.0",
        type=float,
        help="How long to receive messages. (default: %(default)s) "
        "Any negative value means infinite.",
    )
    group3.add_argument(
        "-r",
        "--raw",
        action="store_true",
        help="Use raw mode i.e. send/receive IQ data instead of bits.",
    )

    group4 = parser.add_argument_group("Miscellaneous options")
    group4.add_argument(
        "-h", "--help", action="help", help="show this help message and exit"
    )
    group4.add_argument("-v", "--verbose", action="count")

    return parser


def main():
    def get_val(value, project_params: dict, project_param_key: str, default):
        if value is not None:
            return value
        elif project_param_key in project_params:
            return project_params[project_param_key]
        else:
            return default

    import multiprocessing as mp

    mp.set_start_method("spawn")  # allow usage of prange (OpenMP) in Processes
    mp.freeze_support()

    parser = create_parser()
    args = parser.parse_args()
    if args.parameter_zero is not None or args.parameter_one is not None:
        print(
            "Options -p0 (--parameter-zero) and -p1 (--parameter-one) are not supported anymore.\n"
            "Use --parameters instead e.g. --parameters 20K 40K for a binary FSK."
        )
        sys.exit(1)

    project_params = parse_project_file(args.project_file)
    for argument in ("device", "frequency", "sample_rate"):
        if getattr(args, argument):
            continue

        if project_params[argument] is not None:
            setattr(args, argument, project_params[argument])
        else:
            print("You must specify a {}.".format(argument))
            sys.exit(1)

    if args.receive and args.transmit:
        print("You cannot use receive and transmit mode at the same time.")
        sys.exit(1)
    if not args.receive and not args.transmit:
        print(
            "You must choose a mode either RX (-rx, --receive) or TX (-tx, --transmit)"
        )
        sys.exit(1)

    args.bandwidth = get_val(args.bandwidth, project_params, "bandwidth", None)
    rx_tx_prefix = "rx_" if args.receive else "tx_"
    args.gain = get_val(args.gain, project_params, rx_tx_prefix + "gain", None)
    args.if_gain = get_val(args.if_gain, project_params, rx_tx_prefix + "if_gain", None)
    args.baseband_gain = get_val(
        args.baseband_gain, project_params, rx_tx_prefix + "baseband_gain", None
    )

    if args.modulation_type is None:
        try:
            if project_params["modulation_type"] is None:
                args.modulation_type = MODULATIONS[
                    int(project_params["modulation_index"])
                ]
            else:
                args.modulation_type = project_params["modulation_type"]
        except:
            pass

    if args.bit_length is not None and args.samples_per_symbol is None:
        args.samples_per_symbol = args.bit_length  # legacy
    else:
        args.samples_per_symbol = get_val(
            args.samples_per_symbol,
            project_params,
            "samples_per_symbol",
            DEFAULT_SAMPLES_PER_SYMBOL,
        )

    args.center = get_val(args.center, project_params, "center", DEFAULT_CENTER)
    args.center_spacing = get_val(
        args.center_spacing, project_params, "center_spacing", DEFAULT_CENTER_SPACING
    )
    args.noise = get_val(args.noise, project_params, "noise", DEFAULT_NOISE)
    args.tolerance = get_val(
        args.tolerance, project_params, "tolerance", DEFAULT_TOLERANCE
    )
    args.bits_per_symbol = get_val(
        args.bits_per_symbol, project_params, "bits_per_symbol", 1
    )

    args.carrier_frequency = get_val(
        args.carrier_frequency,
        project_params,
        "carrier_frequency",
        DEFAULT_CARRIER_FREQUENCY,
    )
    args.carrier_amplitude = get_val(
        args.carrier_amplitude,
        project_params,
        "carrier_amplitude",
        DEFAULT_CARRIER_AMPLITUDE,
    )
    args.carrier_phase = get_val(
        args.carrier_phase, project_params, "carrier_phase", DEFAULT_CARRIER_PHASE
    )
    args.parameters = get_val(args.parameters, project_params, "parameters", None)
    if args.parameters is None:
        print("You must give modulation parameters (--parameters)")
        sys.exit(0)

    if args.verbose is None:
        logger.setLevel(logging.ERROR)
    elif args.verbose == 1:
        logger.setLevel(logging.INFO)
    else:
        logger.setLevel(logging.DEBUG)
    Logger.save_log_level()

    argument_string = "\n".join(
        "{} {}".format(arg, getattr(args, arg)) for arg in vars(args)
    )
    logger.debug("Using these parameters\n" + argument_string)

    if args.transmit:
        device = build_device_from_args(args)
        if args.raw:
            if args.filename is None:
                print(
                    "You need to give a file (-file, --filename) where to read samples from."
                )
                sys.exit(1)
            samples_to_send = np.fromfile(args.filename, dtype=np.complex64)
        else:
            modulator = build_modulator_from_args(args)
            messages_to_send = read_messages_to_send(args)
            samples_to_send = modulate_messages(messages_to_send, modulator)
        device.samples_to_send = samples_to_send
        device.start()

        while not device.sending_finished:
            try:
                time.sleep(0.1)
                device.read_messages()
                if device.current_index > 0:
                    cli_progress_bar(
                        device.current_index,
                        len(device.samples_to_send),
                        title="Sending",
                    )
            except KeyboardInterrupt:
                break

        print()
        device.stop("Sending finished")
    elif args.receive:
        if args.raw:
            if args.filename is None:
                print(
                    "You need to give a file (-file, --filename) to receive into when using raw RX mode."
                )
                sys.exit(1)

            receiver = build_device_from_args(args)
            receiver.start()
        else:
            receiver = build_protocol_sniffer_from_args(args)
            receiver.sniff()

        total_time = 0

        if args.receive_time >= 0:
            print("Receiving for {} seconds...".format(args.receive_time))
        else:
            print("Receiving forever...")

        f = None if args.filename is None else open(args.filename, "w")
        kwargs = dict() if f is None else {"file": f}

        dev = receiver.rcv_device if hasattr(receiver, "rcv_device") else receiver

        while total_time < abs(args.receive_time):
            try:
                dev.read_messages()
                time.sleep(0.1)
                if args.receive_time >= 0:
                    # smaller zero means infinity
                    total_time += 0.1

                if not args.raw:
                    num_messages = len(receiver.messages)
                    for msg in receiver.messages[:num_messages]:
                        print(
                            msg.decoded_hex_str if args.hex else msg.decoded_bits_str,
                            **kwargs
                        )
                    del receiver.messages[:num_messages]
            except KeyboardInterrupt:
                break

        print("\nStopping receiving...")
        if args.raw:
            receiver.stop("Receiving finished")
            receiver.data[: receiver.current_index].tofile(f)
        else:
            receiver.stop()

        if f is not None:
            f.close()
            print("Received data written to {}".format(args.filename))


if __name__ == "__main__":
    main()



import math

import numpy as np
from PyQt5.QtCore import QRectF
from PyQt5.QtGui import QPen, QFont, QTransform, QFontMetrics
from PyQt5.QtWidgets import (
    QGraphicsScene,
    QGraphicsRectItem,
    QGraphicsSceneDragDropEvent,
    QGraphicsSimpleTextItem,
)

from urh import settings
from urh.ui.painting.HorizontalSelection import HorizontalSelection
from urh.util import util


class ZoomableScene(QGraphicsScene):
    def __init__(self, parent=None):
        super().__init__(parent)
        self.noise_area = None

        self.separation_areas = []  # type: list[QGraphicsRectItem]
        self.captions = []  # type: list[QGraphicsSimpleTextItem]

        self.centers = [0]

        self.always_show_symbols_legend = False

        self.ones_caption = None
        self.zeros_caption = None

        self.ones_arrow = None
        self.zeros_arrow = None
        self.selection_area = HorizontalSelection(
            0,
            0,
            0,
            0,
            fillcolor=settings.SELECTION_COLOR,
            opacity=settings.SELECTION_OPACITY,
        )
        self.addItem(self.selection_area)

    @property
    def bits_per_symbol(self):
        return int(math.log2(len(self.centers) + 1))

    def draw_noise_area(self, y, h):
        x = self.sceneRect().x()
        w = self.sceneRect().width()

        for area in self.separation_areas:
            area.hide()

        if self.noise_area is None or self.noise_area.scene() != self:
            roi = HorizontalSelection(
                x,
                y,
                w,
                h,
                fillcolor=settings.NOISE_COLOR,
                opacity=settings.NOISE_OPACITY,
            )
            self.noise_area = roi
            self.addItem(self.noise_area)
        else:
            self.noise_area.show()
            self.noise_area.setY(y)
            self.noise_area.height = h

    def hide_legend(self):
        for caption in self.captions:
            caption.hide()

    def clear_legend(self):
        for caption in self.captions:
            self.removeItem(caption)
        self.captions.clear()

    def redraw_legend(self, force_show=False):
        if not (force_show or self.always_show_symbols_legend):
            self.hide_legend()
            return

        num_captions = len(self.centers) + 1
        if num_captions != len(self.captions):
            self.clear_legend()

            fmt = "{0:0" + str(self.bits_per_symbol) + "b}"
            for i in range(num_captions):
                font = QFont()
                font.setPointSize(16)
                font.setBold(True)
                self.captions.append(self.addSimpleText(fmt.format(i), font))

        view_rect = self.parent().view_rect()  # type: QRectF
        padding = 0
        fm = QFontMetrics(self.captions[0].font())

        for i, caption in enumerate(self.captions):
            caption.show()
            scale_x, scale_y = util.calc_x_y_scale(
                self.separation_areas[i].rect(), self.parent()
            )
            try:
                caption.setPos(
                    view_rect.x()
                    + view_rect.width()
                    - fm.width(caption.text()) * scale_x,
                    self.centers[i] + padding,
                )
            except IndexError:
                caption.setPos(
                    view_rect.x()
                    + view_rect.width()
                    - fm.width(caption.text()) * scale_x,
                    self.centers[i - 1] - padding - fm.height() * scale_y,
                )

            caption.setTransform(QTransform.fromScale(scale_x, scale_y), False)

    def draw_sep_area(self, centers: np.ndarray, show_symbols=False):
        x = self.sceneRect().x()
        y = self.sceneRect().y()
        w = self.sceneRect().width()
        h = self.sceneRect().height()
        reversed_centers = list(reversed(centers))

        num_areas = len(centers) + 1
        if num_areas != len(self.separation_areas):
            for area in self.separation_areas:
                self.removeItem(area)
            self.separation_areas.clear()

            for i in range(num_areas):
                area = QGraphicsRectItem(0, 0, 0, 0)
                if i % 2 == 0:
                    area.setBrush(settings.ZEROS_AREA_COLOR)
                else:
                    area.setBrush(settings.ONES_AREA_COLOR)
                area.setOpacity(settings.SEPARATION_OPACITY)
                area.setPen(QPen(settings.TRANSPARENT_COLOR, 0))
                self.addItem(area)
                self.separation_areas.append(area)

        start = y

        for i, area in enumerate(self.separation_areas):
            area.show()
            try:
                self.separation_areas[i].setRect(
                    x, start, w, abs(start - reversed_centers[i])
                )
                start += abs(start - reversed_centers[i])
            except IndexError:
                self.separation_areas[i].setRect(x, start, w, abs(start - h))

        if self.noise_area is not None:
            self.noise_area.hide()

        self.centers = centers
        self.redraw_legend(show_symbols)

    def clear(self):
        self.noise_area = None
        self.separation_areas.clear()
        self.captions.clear()
        self.selection_area = None
        super().clear()

    def dragEnterEvent(self, event: QGraphicsSceneDragDropEvent):
        event.accept()

    def dragMoveEvent(self, event: QGraphicsSceneDragDropEvent):
        event.accept()



import numpy as np
from PyQt5.QtCore import pyqtSlot, pyqtSignal
from PyQt5.QtGui import QIcon, QCloseEvent

from urh.controller.dialogs.SendRecvDialog import SendRecvDialog
from urh.controller.widgets.SniffSettingsWidget import SniffSettingsWidget
from urh.ui.painting.LiveSceneManager import LiveSceneManager
from urh.ui.painting.SniffSceneManager import SniffSceneManager
from urh.util import util


class ProtocolSniffDialog(SendRecvDialog):
    protocol_accepted = pyqtSignal(list)

    def __init__(
        self,
        project_manager,
        signal=None,
        signals=None,
        parent=None,
        testing_mode=False,
    ):
        super().__init__(
            project_manager, is_tx=False, parent=parent, testing_mode=testing_mode
        )

        self.graphics_view = self.ui.graphicsView_sniff_Preview
        self.ui.stackedWidget.setCurrentWidget(self.ui.page_sniff)
        self.hide_send_ui_items()
        self.hide_receive_ui_items()
        self.ui.sliderYscale.hide()
        self.ui.label_y_scale.hide()

        signals = [] if signals is None else signals

        self.sniff_settings_widget = SniffSettingsWidget(
            project_manager=project_manager,
            device_name=self.selected_device_name,
            signal=signal,
            signals=signals,
            backend_handler=self.backend_handler,
        )
        self.ui.scrollAreaWidgetContents_2.layout().insertWidget(
            1, self.sniff_settings_widget
        )
        self.sniff_settings_widget.ui.btn_sniff_use_signal.setAutoDefault(False)

        self.sniffer = self.sniff_settings_widget.sniffer
        self.setWindowTitle(self.tr("Sniff Protocol"))
        self.setWindowIcon(QIcon.fromTheme(":/icons/icons/sniffer.svg"))

        self.ui.txtEd_sniff_Preview.setFont(util.get_monospace_font())

        # set really in on_device_started
        self.scene_manager = None  # type: LiveSceneManager
        self.create_connects()
        self.device_settings_widget.update_for_new_device(overwrite_settings=False)

    @property
    def view_type(self) -> int:
        return self.sniff_settings_widget.ui.comboBox_sniff_viewtype.currentIndex()

    @property
    def show_timestamp(self) -> bool:
        return self.sniff_settings_widget.ui.checkBox_sniff_Timestamp.isChecked()

    def closeEvent(self, event: QCloseEvent):
        self.sniff_settings_widget.emit_sniff_parameters_changed()
        super().closeEvent(event)

    def create_connects(self):
        super().create_connects()
        self.ui.btnAccept.clicked.connect(self.on_btn_accept_clicked)
        self.sniff_settings_widget.sniff_parameters_changed.connect(
            self.device_parameters_changed.emit
        )

        self.sniff_settings_widget.sniff_setting_edited.connect(
            self.on_sniff_setting_edited
        )
        self.sniff_settings_widget.sniff_file_edited.connect(self.on_sniff_file_edited)
        self.sniffer.message_sniffed.connect(self.on_message_sniffed)
        self.sniffer.qt_signals.sniff_device_errors_changed.connect(
            self.on_device_errors_changed
        )

    def init_device(self):
        self.sniffer.device_name = self.selected_device_name
        self.device = self.sniffer.rcv_device

        self._create_device_connects()
        self.scene_manager = SniffSceneManager(
            np.array([], dtype=self.device.data_type), parent=self
        )

    def emit_editing_finished_signals(self):
        super().emit_editing_finished_signals()
        self.sniff_settings_widget.emit_editing_finished_signals()

    def update_view(self):
        if super().update_view():
            self.scene_manager.end = self.device.current_index
            self.scene_manager.init_scene()
            self.scene_manager.show_full_scene()
            self.graphics_view.update()

    @pyqtSlot()
    def on_device_started(self):
        self.scene_manager.data_array = (
            self.device.data.real if hasattr(self.device.data, "real") else None
        )

        super().on_device_started()

        self.ui.btnStart.setEnabled(False)
        self.set_device_ui_items_enabled(False)

    @pyqtSlot()
    def on_sniff_setting_edited(self):
        self.ui.txtEd_sniff_Preview.setPlainText(
            self.sniffer.decoded_to_string(
                self.view_type, include_timestamps=self.show_timestamp
            )
        )

    @pyqtSlot()
    def on_start_clicked(self):
        super().on_start_clicked()
        self.sniffer.sniff()

    @pyqtSlot()
    def on_stop_clicked(self):
        self.sniffer.stop()

    @pyqtSlot()
    def on_clear_clicked(self):
        self.ui.txtEd_sniff_Preview.clear()
        self.scene_manager.clear_path()
        self.device.current_index = 0
        self.sniffer.clear()

    @pyqtSlot(int)
    def on_message_sniffed(self, index: int):
        try:
            msg = self.sniffer.messages[index]
        except IndexError:
            return
        new_data = self.sniffer.message_to_string(
            msg, self.view_type, include_timestamps=self.show_timestamp
        )
        if new_data.strip():
            self.ui.txtEd_sniff_Preview.appendPlainText(new_data)
            self.ui.txtEd_sniff_Preview.verticalScrollBar().setValue(
                self.ui.txtEd_sniff_Preview.verticalScrollBar().maximum()
            )

    @pyqtSlot()
    def on_btn_accept_clicked(self):
        self.protocol_accepted.emit(self.sniffer.messages)
        self.close()

    @pyqtSlot(str)
    def on_device_errors_changed(self, txt: str):
        self.ui.txtEditErrors.append(txt)

    @pyqtSlot()
    def on_sniff_file_edited(self):
        self.ui.btnAccept.setDisabled(bool(self.sniffer.sniff_file))
